------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a real time simulator mainly designed for educational purpose. 
-- This program provides services to automaticly check temporal constraints 
-- of real time tasks.
--
-- Copyright (C) 2002   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the LIMI Team/EA 2215
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------




with Processors;
use Processors;
with Processor_Set;
use Processor_Set;
use Processor_Set.Generic_Processor_Set;
with Task_Set;
use Task_Set;
use Task_Set.Generic_Task_Set;
with Scheduling_Sequences;
use Scheduling_Sequences;

with Ada.Strings.Unbounded;
use  Ada.Strings.Unbounded;


package Call_Scheduling_Stuff is



   type Scheduling_Table is array (Processors_Range) of Scheduling_Sequence; 
   type Period_Table is array (Processors_Range) of Natural; 



   procedure Build_And_Check_Scheduling (
         A_Processor :        Processor_Ptr;      
         Message     : in out Unbounded_String;   
         Period      : in     Natural;            
         Sched       :    out Scheduling_Sequence ); 

   procedure Compute_Scheduling_Period (
         A_Processor :        Processor_Ptr;   
         Message     : in out Unbounded_String ); 

   procedure Compute_Response_Time (
         A_Processor :        Processor_Ptr;   
         Message     : in out Unbounded_String ); 

   procedure Processor_Utilization (
         A_Processor :        Processor_Ptr;   
         Message     : in out Unbounded_String ); 



   pragma Inline (Build_And_Check_Scheduling);
   pragma Inline (Compute_Scheduling_Period);
   pragma Inline (Compute_Response_Time);
   pragma Inline (Processor_Utilization);


end Call_Scheduling_Stuff;


