------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a real time simulator mainly designed for educational purpose. 
-- This program provides services to automaticly check temporal constraints 
-- of real time tasks.
--
-- Copyright (C) 2002   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the LIMI Team/EA 2215
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------



with Glib;
use Glib;
with Gtk;
use Gtk;
with Gdk.Types;
use Gdk.Types;
with Gtk.Widget;
use Gtk.Widget;
with Gtk.Enums;
use Gtk.Enums;

with Callbacks_Cheddar;
use Callbacks_Cheddar;
with Display_Tasks_Set_Pkg.Callbacks;
use Display_Tasks_Set_Pkg.Callbacks;

with Project;
use Project;
with Tasks;
use Tasks;
with Task_Set;
use Task_Set;
use Task_Set.Generic_Task_Set;
with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;
with Config;
use Config;
with Translate;
use Translate;


package body Display_Tasks_Set_Pkg is

   procedure Gtk_New (
         Display_Tasks_Set :    out Display_Tasks_Set_Access ) is 
   begin
      Display_Tasks_Set := new Display_Tasks_Set_Record;
      Display_Tasks_Set_Pkg.Initialize (Display_Tasks_Set);
   end Gtk_New;

   procedure Initialize (
         Display_Tasks_Set : access Display_Tasks_Set_Record'Class ) is 


      A_Task      : Task_Ptr;  
      My_Iterator : Iterator;  
      Ligne       : Guint    := 1;  
      Heigth      : Gint;  

   begin
      Gtk.Window.Initialize (Display_Tasks_Set, Window_Toplevel);
      Set_Title (Display_Tasks_Set,To_String(Lb_Tasks_List(
               Current_Language)));
      Set_Policy (Display_Tasks_Set, True, True, True);
      Heigth:=Gint(3+Get_Number_Of_Elements(Sys.Tasks))*50;
      if Heigth > 400
            then
         Heigth:=400;
      end if;
      Set_Default_Size (Display_Tasks_Set, 600, Heigth);
      Set_Position (Display_Tasks_Set, Win_Pos_Center_Always);
      Set_Modal (Display_Tasks_Set, True);
      Set_Extension_Events (Display_Tasks_Set, Extension_Events_All);
      Set_Flags (Display_Tasks_Set, Can_Default);
      Grab_Focus (Display_Tasks_Set);


      Gtk_New (Display_Tasks_Set.Scroll);
      Set_Border_Width (Display_Tasks_Set.Scroll, Border_Width => 10);
      Set_Policy (Display_Tasks_Set.Scroll,
         H_Scrollbar_Policy => Policy_Automatic,
         V_Scrollbar_Policy => Policy_Automatic);
      Add (Display_Tasks_Set, Display_Tasks_Set.Scroll);


      Gtk_New (Display_Tasks_Set.Table, Guint(3+Get_Number_Of_Elements(
               Sys.Tasks)), 7, True);
      Set_Row_Spacings (Display_Tasks_Set.Table, 8);
      Set_Col_Spacings (Display_Tasks_Set.Table, 7);


      Add_With_Viewport (Display_Tasks_Set.Scroll, Display_Tasks_Set.
         Table);
      Set_Focus_Hadjustment (Display_Tasks_Set.Table, Get_Hadjustment (
            Display_Tasks_Set.Scroll));
      Set_Focus_Vadjustment (Display_Tasks_Set.Table, Get_Vadjustment (
            Display_Tasks_Set.Scroll));


      Gtk_New (Display_Tasks_Set.Name, To_String(Lb_Task_Name(
               Current_Language)));
      Set_Alignment (Display_Tasks_Set.Name, 0.5, 0.5);
      Set_Padding (Display_Tasks_Set.Name, 0, 0);
      Set_Justify (Display_Tasks_Set.Name, Justify_Center);
      Set_Line_Wrap (Display_Tasks_Set.Name, False);
      Attach (Display_Tasks_Set.Table, Display_Tasks_Set.Name, 0, 1, 0,
         1,
         0, 0,
         0, 0);

      Gtk_New (Display_Tasks_Set.Processor, To_String(Lb_Processor(
               Current_Language)));
      Set_Alignment (Display_Tasks_Set.Processor, 0.5, 0.5);
      Set_Padding (Display_Tasks_Set.Processor, 0, 0);
      Set_Justify (Display_Tasks_Set.Processor, Justify_Center);
      Set_Line_Wrap (Display_Tasks_Set.Processor, False);
      Attach (Display_Tasks_Set.Table, Display_Tasks_Set.Processor, 1, 2,
         0,
         1,
         0, 0,
         0, 0);

      Gtk_New (Display_Tasks_Set.Start_Time, To_String(Lb_Start_Time(
               Current_Language)));
      Set_Alignment (Display_Tasks_Set.Start_Time, 0.5, 0.5);
      Set_Padding (Display_Tasks_Set.Start_Time, 0, 0);
      Set_Justify (Display_Tasks_Set.Start_Time, Justify_Center);
      Set_Line_Wrap (Display_Tasks_Set.Start_Time, False);
      Attach (Display_Tasks_Set.Table, Display_Tasks_Set.Start_Time, 2, 3,
         0,
         1,
         0, 0,
         0, 0);

      Gtk_New (Display_Tasks_Set.Capacity, To_String(Lb_Capacity(
               Current_Language)));
      Set_Alignment (Display_Tasks_Set.Capacity, 0.5, 0.5);
      Set_Padding (Display_Tasks_Set.Capacity, 0, 0);
      Set_Justify (Display_Tasks_Set.Capacity, Justify_Center);
      Set_Line_Wrap (Display_Tasks_Set.Capacity, False);
      Attach (Display_Tasks_Set.Table, Display_Tasks_Set.Capacity, 3, 4, 0,
         1,
         0, 0,
         0, 0);

      Gtk_New (Display_Tasks_Set.Period, To_String(Lb_Period(
               Current_Language)));
      Set_Alignment (Display_Tasks_Set.Period, 0.5, 0.5);
      Set_Padding (Display_Tasks_Set.Period, 0, 0);
      Set_Justify (Display_Tasks_Set.Period, Justify_Center);
      Set_Line_Wrap (Display_Tasks_Set.Period, False);
      Attach (Display_Tasks_Set.Table, Display_Tasks_Set.Period, 4, 5, 0,
         1,
         0, 0,
         0, 0);


      Gtk_New (Display_Tasks_Set.Deadline, To_String(Lb_Deadline(
               Current_Language)));
      Set_Alignment (Display_Tasks_Set.Deadline, 0.5, 0.5);
      Set_Padding (Display_Tasks_Set.Deadline, 0, 0);
      Set_Justify (Display_Tasks_Set.Deadline, Justify_Center);
      Set_Line_Wrap (Display_Tasks_Set.Deadline, False);
      Attach (Display_Tasks_Set.Table, Display_Tasks_Set.Deadline, 5, 6, 0,
         1,
         0, 0,
         0, 0);

      Gtk_New (Display_Tasks_Set.Jitter, To_String(Lb_Jitter(
               Current_Language)));
      Set_Alignment (Display_Tasks_Set.Jitter, 0.5, 0.5);
      Set_Padding (Display_Tasks_Set.Jitter, 0, 0);
      Set_Justify (Display_Tasks_Set.Jitter, Justify_Center);
      Set_Line_Wrap (Display_Tasks_Set.Jitter, False);
      Attach (Display_Tasks_Set.Table, Display_Tasks_Set.Jitter, 6, 7, 0,
         1,
         0, 0,
         0, 0);

      Gtk_New (Display_Tasks_Set.Priority, To_String(Lb_Priority(
               Current_Language)));
      Set_Alignment (Display_Tasks_Set.Priority, 0.5, 0.5);
      Set_Padding (Display_Tasks_Set.Priority, 0, 0);
      Set_Justify (Display_Tasks_Set.Priority, Justify_Center);
      Set_Line_Wrap (Display_Tasks_Set.Priority, False);
      Attach (Display_Tasks_Set.Table, Display_Tasks_Set.Priority, 7, 8, 0,
         1,
         0, 0,
         0, 0);


      Gtk_New (Display_Tasks_Set.Priority, To_String(Lb_Offset_Act(
               Current_Language)));
      Set_Alignment (Display_Tasks_Set.Priority, 0.5, 0.5);
      Set_Padding (Display_Tasks_Set.Priority, 0, 0);
      Set_Justify (Display_Tasks_Set.Priority, Justify_Center);
      Set_Line_Wrap (Display_Tasks_Set.Priority, False);
      Attach (Display_Tasks_Set.Table, Display_Tasks_Set.Priority,  8, 11,
         0,
         1,
         0, 0,
         0, 0);


      Gtk_New (Display_Tasks_Set.Priority, To_String(Lb_Offset_Val(
               Current_Language)));
      Set_Alignment (Display_Tasks_Set.Priority, 0.5, 0.5);
      Set_Padding (Display_Tasks_Set.Priority, 0, 0);
      Set_Justify (Display_Tasks_Set.Priority, Justify_Center);
      Set_Line_Wrap (Display_Tasks_Set.Priority, False);
      Attach (Display_Tasks_Set.Table, Display_Tasks_Set.Priority, 11, 14,
         0,
         1,
         0, 0,
         0, 0);



      Reset_Iterator(My_Iterator);

      loop
         Current_Element(Sys.Tasks, A_Task, My_Iterator);

         Gtk_New (Display_Tasks_Set.Entry_Name);
         Set_Editable (Display_Tasks_Set.Entry_Name, False);
         Set_Max_Length (Display_Tasks_Set.Entry_Name, 0);
         Set_Text (Display_Tasks_Set.Entry_Name, To_String(A_Task.Name) );
         Set_Visibility (Display_Tasks_Set.Entry_Name, True);
         Attach (Display_Tasks_Set.Table, Display_Tasks_Set.Entry_Name, 0,
            1,
            Ligne,
            Ligne+1,
            Expand or Fill, 0,
            0, 0);

         Gtk_New (Display_Tasks_Set.Entry_Processor);
         Set_Editable (Display_Tasks_Set.Entry_Processor, False);
         Set_Max_Length (Display_Tasks_Set.Entry_Processor, 0);
         Set_Text (Display_Tasks_Set.Entry_Processor, To_String(A_Task.
               Location));
         Set_Visibility (Display_Tasks_Set.Entry_Processor, True);
         Attach (Display_Tasks_Set.Table, Display_Tasks_Set.
            Entry_Processor, 1, 2,
            Ligne,
            Ligne+1,
            Expand or Fill, 0,
            0, 0);

         Gtk_New (Display_Tasks_Set.Entry_Start_Time);
         Set_Editable (Display_Tasks_Set.Entry_Start_Time, False);
         Set_Max_Length (Display_Tasks_Set.Entry_Start_Time, 0);
         Set_Text (Display_Tasks_Set.Entry_Start_Time, A_Task.Start_Time'
            Img);
         Set_Visibility (Display_Tasks_Set.Entry_Start_Time, True);
         Attach (Display_Tasks_Set.Table, Display_Tasks_Set.
            Entry_Start_Time, 2, 3,
            Ligne,
            Ligne+1,
            Expand or Fill, 0,
            0, 0);

         Gtk_New (Display_Tasks_Set.Entry_Capacity);
         Set_Editable (Display_Tasks_Set.Entry_Capacity, False);
         Set_Max_Length (Display_Tasks_Set.Entry_Capacity, 0);
         Set_Text (Display_Tasks_Set.Entry_Capacity, A_Task.Capacity'Img);
         Set_Visibility (Display_Tasks_Set.Entry_Capacity, True);
         Attach (Display_Tasks_Set.Table, Display_Tasks_Set.
            Entry_Capacity, 3, 4,
            Ligne,
            Ligne+1,
            Expand or Fill, 0,
            0, 0);

         Gtk_New (Display_Tasks_Set.Entry_Period);
         Set_Editable (Display_Tasks_Set.Entry_Period, False);
         Set_Max_Length (Display_Tasks_Set.Entry_Period, 0);
         if (A_Task.Task_Type = Periodic_Type)
               then
            Set_Text (Display_Tasks_Set.Entry_Period, Periodic_Task_Ptr(
                  A_Task).Period'Img);
         else
            Set_Text (Display_Tasks_Set.Entry_Period, "0");
         end if;

         Set_Visibility (Display_Tasks_Set.Entry_Period, True);
         Attach (Display_Tasks_Set.Table, Display_Tasks_Set.Entry_Period,
            4, 5,
            Ligne,
            Ligne+1,
            Expand or Fill, 0,
            0, 0);

         Gtk_New (Display_Tasks_Set.Entry_Deadline);
         Set_Editable (Display_Tasks_Set.Entry_Deadline, False);
         Set_Max_Length (Display_Tasks_Set.Entry_Deadline, 0);
         if (A_Task.Task_Type = Periodic_Type)
               then
            Set_Text (Display_Tasks_Set.Entry_Deadline, Periodic_Task_Ptr(
                  A_Task).
               Deadline'Img);
         else
            Set_Text (Display_Tasks_Set.Entry_Deadline, Aperiodic_Task_Ptr(
                  A_Task).
               Deadline'Img);
         end if;
         Set_Visibility (Display_Tasks_Set.Entry_Deadline, True);
         Attach (Display_Tasks_Set.Table, Display_Tasks_Set.
            Entry_Deadline, 5, 6,
            Ligne,
            Ligne+1,
            Expand or Fill, 0,
            0, 0);


         Gtk_New (Display_Tasks_Set.Entry_Jitter);
         Set_Editable (Display_Tasks_Set.Entry_Jitter, False);
         Set_Max_Length (Display_Tasks_Set.Entry_Jitter, 0);
         if (A_Task.Task_Type = Periodic_Type)
               then
            Set_Text (Display_Tasks_Set.Entry_Jitter, Periodic_Task_Ptr(
                  A_Task).
               Jitter'Img);
         else
            Set_Text (Display_Tasks_Set.Entry_Jitter, "0");
         end if;
         Set_Visibility (Display_Tasks_Set.Entry_Jitter, True);
         Attach (Display_Tasks_Set.Table, Display_Tasks_Set.Entry_Jitter,
            6, 7,
            Ligne,
            Ligne+1,
            Expand or Fill, 0,
            0, 0);

         Gtk_New (Display_Tasks_Set.Entry_Priority);
         Set_Editable (Display_Tasks_Set.Entry_Priority, False);
         Set_Max_Length (Display_Tasks_Set.Entry_Priority, 0);
         Set_Text (Display_Tasks_Set.Entry_Priority, A_Task.Priority'Img);
         Set_Visibility (Display_Tasks_Set.Entry_Priority, True);
         Attach (Display_Tasks_Set.Table, Display_Tasks_Set.
            Entry_Priority, 7, 8,
            Ligne,
            Ligne+1,
            Expand or Fill, 0,
            0, 0);

         Gtk_New (Display_Tasks_Set.Entry_Offset1_Act);
         Set_Editable (Display_Tasks_Set.Entry_Offset1_Act, False);
         Set_Max_Length (Display_Tasks_Set.Entry_Offset1_Act, 0);
         Set_Text (Display_Tasks_Set.Entry_Offset1_Act, A_Task.Offset(1).
            Activation'Img);
         Set_Visibility (Display_Tasks_Set.Entry_Offset1_Act, True);
         Attach (Display_Tasks_Set.Table, Display_Tasks_Set.
            Entry_Offset1_Act, 8, 9,
            Ligne,
            Ligne+1,
            Expand or Fill, 0,
            0, 0);

         Gtk_New (Display_Tasks_Set.Entry_Offset2_Act);
         Set_Editable (Display_Tasks_Set.Entry_Offset2_Act, False);
         Set_Max_Length (Display_Tasks_Set.Entry_Offset2_Act, 0);
         Set_Text (Display_Tasks_Set.Entry_Offset2_Act, A_Task.Offset(2).
            Activation'Img);
         Set_Visibility (Display_Tasks_Set.Entry_Offset2_Act, True);
         Attach (Display_Tasks_Set.Table, Display_Tasks_Set.
            Entry_Offset2_Act,  9, 10,
            Ligne,
            Ligne+1,
            Expand or Fill, 0,
            0, 0);

         Gtk_New (Display_Tasks_Set.Entry_Offset3_Act);
         Set_Editable (Display_Tasks_Set.Entry_Offset3_Act, False);
         Set_Max_Length (Display_Tasks_Set.Entry_Offset3_Act, 0);
         Set_Text (Display_Tasks_Set.Entry_Offset3_Act, A_Task.Offset(3).
            Activation'Img);
         Set_Visibility (Display_Tasks_Set.Entry_Offset3_Act, True);
         Attach (Display_Tasks_Set.Table, Display_Tasks_Set.
            Entry_Offset3_Act, 10, 11,
            Ligne,
            Ligne+1,
            Expand or Fill, 0,
            0, 0);

         Gtk_New (Display_Tasks_Set.Entry_Offset1_Val);
         Set_Editable (Display_Tasks_Set.Entry_Offset1_Val, False);
         Set_Max_Length (Display_Tasks_Set.Entry_Offset1_Val, 0);
         Set_Text (Display_Tasks_Set.Entry_Offset1_Val, A_Task.Offset(1).
            Value'Img);
         Set_Visibility (Display_Tasks_Set.Entry_Offset1_Val, True);
         Attach (Display_Tasks_Set.Table, Display_Tasks_Set.
            Entry_Offset1_Val, 11, 12,
            Ligne,
            Ligne+1,
            Expand or Fill, 0,
            0, 0);

         Gtk_New (Display_Tasks_Set.Entry_Offset2_Val);
         Set_Editable (Display_Tasks_Set.Entry_Offset2_Val, False);
         Set_Max_Length (Display_Tasks_Set.Entry_Offset2_Val, 0);
         Set_Text (Display_Tasks_Set.Entry_Offset2_Val, A_Task.Offset(2).
            Value'Img);
         Set_Visibility (Display_Tasks_Set.Entry_Offset2_Val, True);
         Attach (Display_Tasks_Set.Table, Display_Tasks_Set.
            Entry_Offset2_Val, 12, 13,
            Ligne,
            Ligne+1,
            Expand or Fill, 0,
            0, 0);


         Gtk_New (Display_Tasks_Set.Entry_Offset3_Val);
         Set_Editable (Display_Tasks_Set.Entry_Offset3_Val, False);
         Set_Max_Length (Display_Tasks_Set.Entry_Offset3_Val, 0);
         Set_Text (Display_Tasks_Set.Entry_Offset3_Val, A_Task.Offset(3).
            Value'Img);
         Set_Visibility (Display_Tasks_Set.Entry_Offset3_Val, True);
         Attach (Display_Tasks_Set.Table, Display_Tasks_Set.
            Entry_Offset3_Val, 13, 14,
            Ligne,
            Ligne+1,
            Expand or Fill, 0,
            0, 0);





         Ligne:=Ligne+1;
         exit when Is_Last_Element(Sys.Tasks, My_Iterator) = True;

         Next_Element(Sys.Tasks, My_Iterator);

      end loop;




      Gtk_New_Hseparator (Display_Tasks_Set.Separator);
      Attach (Display_Tasks_Set.Table, Display_Tasks_Set.Separator, 0,
         8, Ligne, Ligne+1,
         Fill, Expand or Fill,
         0, 0);

      Ligne:=Ligne+1;
      Gtk_New (Display_Tasks_Set.Ok_Button, "OK");
      Set_Usize (Display_Tasks_Set.Ok_Button, 70, 25);
      Attach (Display_Tasks_Set.Table, Display_Tasks_Set.Ok_Button, 0, 14,
         Ligne, Ligne+1,
         0, 0,
         0, 0);
      Button_Callback.Connect
         (Display_Tasks_Set.Ok_Button, "clicked",
         Button_Callback.To_Marshaller (On_Ok_Pressed'access));



   end Initialize;

end Display_Tasks_Set_Pkg;
