------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a real time simulator mainly designed for educational purpose. 
-- This program provides services to automaticly check temporal constraints 
-- of real time tasks.
--
-- Copyright (C) 2002   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the LIMI Team/EA 2215
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------



with Glib;
use Glib;
with Gtk.Check_Button;
use Gtk.Check_Button;

with Ihm_Tools;
use Ihm_Tools;

with Project;
use Project;
with Processor_Set;
use Processor_Set;
use Processor_Set.Generic_Processor_Set;

with Scheduler;
use Scheduler;
with Scheduler.Edf;
use Scheduler.Edf;
with Scheduler.Rm;
use Scheduler.Rm;
with Scheduler.Llf;
use Scheduler.Llf;
with Scheduler.Dm;
use Scheduler.Dm;
with Scheduler.Hpf;
use Scheduler.Hpf;

with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;

with Gtk.Radio_Menu_Item;
use Gtk.Radio_Menu_Item;

with Config;
use Config;

with Translate;
use Translate;


package body Processors_Parameters_Pkg.Callbacks is




   procedure On_Ok_Pressed (
         Object : access Gtk_Button_Record'Class ) is 

      Name        : Unbounded_String;  
      Sched       : Gint;  
      A_Scheduler : Scheduler_Ptr;  


   begin

      Name:=To_Unbounded_String(Get_Text(Processors_Parameters.Entry_Name));

      if Name=""
            then
         Show_Message_Box(
            Lb_Processor_Name(Current_Language) &
            Lb_Mandatory(Current_Language) );
         return;
      end if;



      Sched:= Gint'Val (4 -
         Gint(Selected_Button (Processors_Parameters.Clist_Scheduler_Menu)));


      if (Sched=0)
            then
         A_Scheduler := new Edf_Scheduler;
      end if;
      if (Sched=1)
            then
         A_Scheduler := new Llf_Scheduler;
      end if;
      if (Sched=2)
            then
         A_Scheduler := new Rm_Scheduler;
      end if;
      if (Sched=3)
            then
         A_Scheduler := new Dm_Scheduler;
      end if;
      if (Sched=4)
            then
         A_Scheduler := new Hpf_Scheduler;
      end if;


      if Get_Active(Processors_Parameters.Entry_Option)
            then
         Set_Preemptive(A_Scheduler.all, Preemptive);
      else
         Set_Preemptive(A_Scheduler.all, Non_Preemptive);
      end if;

      Add_Processor(Sys.Processors,Name,A_Scheduler);

      Is_New:=False;
      Is_Saved:=False;
      Hide(Processors_Parameters);

   exception
      when Processor_Already_Defined =>
         begin
            Show_Message_Box(Lb_Processor_Name(Current_Language)
               & Lb_Already_Defined(Current_Language));
            return;
         end;


   end On_Ok_Pressed;




   procedure On_Cancel_Pressed (
         Object : access Gtk_Button_Record'Class ) is 
   begin
      Hide(Processors_Parameters);
   end On_Cancel_Pressed;

end Processors_Parameters_Pkg.Callbacks;
