------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a real time simulator mainly designed for educational purpose. 
-- This program provides services to automaticly check temporal constraints 
-- of real time tasks.
--
-- Copyright (C) 2002   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the LIMI Team/EA 2215
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------



with Gtk;
use Gtk;
with Gtk.Main;
use Gtk.Main;


with Ihm_Tools;
use Ihm_Tools;

with Open_File_Selection_Pkg;
use Open_File_Selection_Pkg;
with Save_File_Selection_Pkg;
use Save_File_Selection_Pkg;
with Root_Window_Pkg.Callbacks;
use Root_Window_Pkg.Callbacks;

with Config;
use Config;
with Translate;
use Translate;


package body Project is


   procedure Do_New is 
   begin
      Systems.Reset(Sys);
      File_Name:=To_Unbounded_String("");
      Is_Saved:=False;
      Is_New:=True;
   end Do_New;

   procedure Do_Open is 
   begin
      Clear_Work_Space;
      if (Is_Saved = False)
            then
         if not Is_New
               then
            Show_Message_Box(Lb_Not_Saved(Current_Language));
         else
            Gtk_New (Open_File_Selection);
            Show_All (Open_File_Selection);
         end if;
      else
         Gtk_New (Open_File_Selection);
         Show_All (Open_File_Selection);
      end if;
   end  Do_Open;

   procedure Do_Save is 
   begin
      if (Is_Saved = False)
            then
         if (File_Name /= To_Unbounded_String(""))
               then
            Systems.Write_To_File(Sys, File_Name);
            Is_Saved:=True;
         else
            Gtk_New (Save_File_Selection);
            Show_All (Save_File_Selection);
         end if;
      end if;
   end  Do_Save;

   procedure Do_Save_As is 
   begin
      Gtk_New (Save_File_Selection);
      Show_All (Save_File_Selection);
   end  Do_Save_As;

   procedure Do_Exit is 
   begin
      if (Is_Saved = False)
            then
         if (not Is_New)
               then
            Show_Message_Box(Lb_Not_Saved(Current_Language));
            return;
         end if;
      end if;

      Gtk.Main.Gtk_Exit (0);
   end  Do_Exit;


end Project;

