------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a real time simulator mainly designed for educational purpose. 
-- This program provides services to automaticly check temporal constraints 
-- of real time tasks.
--
-- Copyright (C) 2002   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the LIMI Team/EA 2215
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------



with Config;
use Config;

with Translate;
use Translate;

with Project;
use Project;

with References;
with Manual;

with Processors;
use Processors;
with Processor_Set;
use Processor_Set;
use Processor_Set.Generic_Processor_Set;

with Tasks;
use Tasks;
with Task_Set;
use Task_Set;
use Task_Set.Generic_Task_Set;

with Scheduler;
use Scheduler;

with Call_Scheduling_Stuff;
use Call_Scheduling_Stuff;

with Scheduling_Sequences;
use Scheduling_Sequences;

with Glib;
use Glib;
with Gtk.Widget;
use Gtk.Widget;

with Ihm_Tools;
use Ihm_Tools;

with Tasks_Parameters_Pkg;
use Tasks_Parameters_Pkg;
with Processors_Parameters_Pkg;
use Processors_Parameters_Pkg;
with Display_Tasks_Set_Pkg;
use Display_Tasks_Set_Pkg;
with Update_Tasks_Set_Pkg;
use Update_Tasks_Set_Pkg;
with Display_Processors_Set_Pkg;
use Display_Processors_Set_Pkg;
with Update_Processors_Set_Pkg;
use Update_Processors_Set_Pkg;
with Draw_Scheduling_Pkg;
use Draw_Scheduling_Pkg;
with Scrolling_Text_Pkg;
use Scrolling_Text_Pkg;
with Question_Pkg;
use Question_Pkg;
with Unbounded_Strings;
use Unbounded_Strings;



package body Root_Window_Pkg.Callbacks is

   use Gtk.Arguments;


   procedure Clear_Work_Space is 
      Size        : Guint;  
      I_Dont_Care : Boolean;  
   begin
      Size:=Get_Length(Root_Window.Text);
      Set_Point(Root_Window.Text,0);
      I_Dont_Care:=Forward_Delete(Root_Window.Text,Size);
      Clear_Task(Draw_Scheduling);
   end Clear_Work_Space;


   function Delete_Processor_Callback (
         Value : Unbounded_String ) 
     return Boolean is 

      Iterator1   : Processors_Iterator;  
      A_Processor : Processor_Ptr;  
      Iterator2   : Tasks_Iterator;  
      A_Task      : Task_Ptr;  
      Nb          : Tasks_Range;  

   begin

      Reset_Iterator(Iterator1);

      loop

         Current_Element(Sys.Processors, A_Processor, Iterator1);

         if A_Processor.Name = Value
               then
            Delete(Sys.Processors,A_Processor);

            -- delete task of this processor
            --

            Nb:=Get_Number_Of_Task_From_Processor(Sys.Tasks,Value);

            for I in  1..Nb loop

               Reset_Iterator(Iterator2);
               loop
                  Current_Element(Sys.Tasks, A_Task, Iterator2);

                  if A_Task.Location=Value
                        then
                     Delete(Sys.Tasks, A_Task);
                     Is_Saved:=False;

                     exit;
                  end if;

                  Next_Element(Sys.Tasks, Iterator2);
               end loop;

            end loop;
            return True;
         end if;

         exit when Is_Last_Element(Sys.Processors, Iterator1) = True;
         Next_Element(Sys.Processors, Iterator1);

      end loop;


      Show_Message_Box(Lb_Processor_Not_Found(Current_Language));
      return False;

   end Delete_Processor_Callback;


   function Delete_Task_Callback (
         Value : Unbounded_String ) 
     return Boolean is 

      My_Iterator : Tasks_Iterator;  
      A_Task      : Task_Ptr;  
   begin

      loop

         Current_Element(Sys.Tasks, A_Task, My_Iterator);

         if A_Task.Name = Value
               then
            Delete(Sys.Tasks,A_Task);
            Is_Saved:=False;
            -- hide the window
            return True;
         end if;

         exit when Is_Last_Element(Sys.Tasks, My_Iterator) = True;
         Next_Element(Sys.Tasks, My_Iterator);

      end loop;


      Show_Message_Box(Lb_Task_Not_Found(Current_Language));
      return False;

   end Delete_Task_Callback;


   procedure Check_Scheduling is 

      A_Processor : Processor_Ptr;  
      My_Iterator : Processors_Iterator;  
      Msg         : Unbounded_String;  

   begin

      if (Get_Number_Of_Elements(Sys.Tasks)=0)
            then
         Show_Message_Box(Lb_Define_Tasks_Before(Current_Language));
         return;
      end if;

      Write_Text(Root_Window, Unbounded_Lf & Unbounded_Lf & Unbounded_Lf);

      Reset_Iterator(My_Iterator);

      loop
         Current_Element(Sys.Processors, A_Processor, My_Iterator);


         if (Get_Number_Of_Task_From_Processor(Sys.Tasks, A_Processor.
                  Name)/=0)
               then

            Write_Title(Root_Window, Lb_Processor(Current_Language)  &
               A_Processor.Name  & To_Unbounded_String(" : ") &
               Unbounded_Lf & Unbounded_Lf);

            Msg:=To_Unbounded_String("");
            Compute_Scheduling_Period(A_Processor, Msg);
            Write_Text(Root_Window, Msg);

            Msg:=To_Unbounded_String("");
            Processor_Utilization(A_Processor, Msg);
            Write_Text(Root_Window, Msg);

            Msg:=To_Unbounded_String("");
            Compute_Response_Time(A_Processor, Msg);
            Write_Text(Root_Window, Msg);

            Write_Text(Root_Window, Unbounded_Lf & Unbounded_Lf);

         end if;

         exit when Is_Last_Element(Sys.Processors, My_Iterator) = True;

         Next_Element(Sys.Processors, My_Iterator);

      end loop;

   end Check_Scheduling;




   function Draw_Scheduling_Callback (
         Value : Unbounded_String ) 
     return Boolean is 

      Sched       : Scheduling_Table;  
      Sched1      : Scheduling_Sequence;  
      A_Processor : Processor_Ptr;  
      My_Iterator : Processors_Iterator;  
      Msg         : Unbounded_String;  
      Index       : Processors_Range    := 0;  
      Last_Period : Natural             := Config.Max_Scheduling_Period -
        1;  
      Max_Period  : Natural             := 0;  
      Ok          : Boolean;  

   begin


      Ok:=True;
      To_Natural(Value,Max_Period,Ok);
      if not Ok
            then
         Show_Message_Box(Lb_Parameter(Current_Language) &
            Lb_Must_Be_Numeric(Current_Language));
         return False;
      end if;

      if(Max_Period>=Last_Period)
            then
         Show_Message_Box(Lb_Parameter(Current_Language) & Lb_Must_Be(
               Current_Language) & To_Unbounded_String(" < ")
            & Last_Period'Img);
         return False;
      end if;

      if(Max_Period=0)
            then
         Show_Message_Box(Lb_Parameter(Current_Language) &
            Lb_Must_Be(Current_Language) & To_Unbounded_String(" > 0"));
         return False;
      end if;


      Write_Text(Root_Window, Unbounded_Lf & Unbounded_Lf & Unbounded_Lf);

      Reset_Iterator(My_Iterator);

      loop
         Current_Element(Sys.Processors, A_Processor, My_Iterator);

         if (Get_Number_Of_Task_From_Processor(Sys.Tasks, A_Processor.
                  Name)/=0)
               then

            Write_Title(Root_Window, Lb_Processor(Current_Language) &
               A_Processor.Name  & To_Unbounded_String(" : ") &
               Unbounded_Lf & Unbounded_Lf);

            Msg:=To_Unbounded_String("");
            Build_And_Check_Scheduling(A_Processor, Msg, Max_Period,
               Sched1);
            Sched(Index):=Sched1;
            Index:=Index+1;

            Write_Text(Root_Window, Msg);
            Write_Text(Root_Window, Unbounded_Lf & Unbounded_Lf);


         end if;

         exit when Is_Last_Element(Sys.Processors, My_Iterator) = True;

         Next_Element(Sys.Processors, My_Iterator);

      end loop;

      Draw_Task(Draw_Scheduling,Sys.Tasks,Max_Period,Sched);


      return True;

   end Draw_Scheduling_Callback;



   procedure Schedule_And_Check_Temporal_Constraints is 


      A_Processor : Processor_Ptr;  
      My_Iterator : Processors_Iterator;  
      Max_Period  : Natural             := 0;  
      Period      : Natural             := 0;  
      Last_Period : Natural             := Config.Max_Scheduling_Period -
        1;  

   begin

      if (Get_Number_Of_Elements(Sys.Tasks)=0)
            then
         Show_Message_Box(
            Lb_Define_Tasks_Before(Current_Language));
         return;
      end if;


      Reset_Iterator(My_Iterator);

      loop


         Current_Element(Sys.Processors, A_Processor, My_Iterator);

         if (Get_Number_Of_Task_From_Processor(Sys.Tasks, A_Processor.
                  Name)/=0)
               then

            begin
               Period:=Scheduling_Period(A_Processor.Scheduler.all, Sys.
                  Tasks, A_Processor.Name);
               if Period> Max_Period
                     then
                  Max_Period:=Period;
               end if;

            exception
               when Scheduler.Task_Must_Be_Periodic =>
                  null;
               when Scheduler.Task_Model_Error =>
                  null;

            end;
         end if;

         exit when Is_Last_Element(Sys.Processors, My_Iterator) = True;

         Next_Element(Sys.Processors, My_Iterator);


      end loop;


      if(Max_Period>=Last_Period)
            then
         Max_Period:=Last_Period;
      end if;

      Gtk_New(Question, Lb_Draw(Current_Language), Lb_Schedule_From_To(
            Current_Language),
         To_Unbounded_String(Max_Period'Img),
         Draw_Scheduling_Callback'access);
      Show_All(Question);

   end Schedule_And_Check_Temporal_Constraints;


   function On_Root_Window_Delete_Event (
         Object : access Gtk_Widget_Record'Class; 
         Params :        Gtk.Arguments.Gtk_Args   ) 
     return Boolean is 
   begin
      Do_Exit;
      return True;
   end On_Root_Window_Delete_Event;


   procedure On_File_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin
      null;
   end On_File_Activate;

   procedure On_New_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin
      Clear_Work_Space;
      Do_New;
   end On_New_Activate;

   procedure On_Open_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin
      Do_Open;
   end On_Open_Activate;


   procedure On_Save_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin
      Do_Save;
   end On_Save_Activate;


   procedure On_Save_As_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin
      Do_Save_As;
   end On_Save_As_Activate;


   procedure On_Exit_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin
      Do_Exit;
   end On_Exit_Activate;

   procedure On_New_Task_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin
      if (Get_Number_Of_Elements(Sys.Processors)=0)
            then
         Show_Message_Box(Lb_Define_Processors_Before(Current_Language));
      else
         Gtk_New (Tasks_Parameters);
         Show_All (Tasks_Parameters);
      end if;
   end On_New_Task_Activate;

   procedure On_Update_Task_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin
      if (Get_Number_Of_Elements(Sys.Tasks)=0)
            then
         Show_Message_Box(Lb_Define_Tasks_Before(Current_Language));
      else
         Gtk_New (Update_Tasks_Set);
         Show_All (Update_Tasks_Set);
      end if;
   end On_Update_Task_Activate;

   procedure On_New_Processor_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin
      Gtk_New (Processors_Parameters);
      Show_All (Processors_Parameters);
   end On_New_Processor_Activate;

   procedure On_Update_Processor_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin
      if (Get_Number_Of_Elements(Sys.Processors)=0)
            then
         Show_Message_Box(Lb_Define_Processors_Before(Current_Language));
      else
         Gtk_New (Update_Processors_Set);
         Show_All (Update_Processors_Set);
      end if;
   end On_Update_Processor_Activate;

   procedure On_Tasks_List_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin

      if (Get_Number_Of_Elements(Sys.Tasks)=0)
            then
         Show_Message_Box(Lb_Define_Tasks_Before(Current_Language));
      else
         Gtk_New (Display_Tasks_Set);
         Show_All (Display_Tasks_Set);
      end if;
   end On_Tasks_List_Activate;


   procedure On_Processors_List_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin
      if (Get_Number_Of_Elements(Sys.Processors)=0)
            then
         Show_Message_Box(Lb_Define_Processors_Before(Current_Language));
      else
         Gtk_New (Display_Processors_Set);
         Show_All (Display_Processors_Set);
      end if;
   end On_Processors_List_Activate;


   procedure On_Tools_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin
      null;
   end On_Tools_Activate;


   procedure On_Clear_Work_Space_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin
      Clear_Work_Space;
   end On_Clear_Work_Space_Activate;


   procedure On_Draw_Scheduling_Graph_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 

   begin
      Schedule_And_Check_Temporal_Constraints;
   end On_Draw_Scheduling_Graph_Activate;


   procedure On_Check_Scheduling_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin
      Check_Scheduling;
   end On_Check_Scheduling_Activate;


   procedure On_Help_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin
      null;
   end On_Help_Activate;


   procedure On_About_Cheddar_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 

   begin
      Show_Message_Box(Lb_About_Text(Current_Language));
   end On_About_Cheddar_Activate;



   procedure On_Manual_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin

      Gtk_New(Scrolling_Text,Lb_Manual(Current_Language));
      Set_Usize(Scrolling_Text,690,425);


      Write_Title_Small(Scrolling_Text,Manual.Ustrfilemenu);
      Write_Text(Scrolling_Text,Manual.Ustrfilemenucontent);

      Write_Title_Small(Scrolling_Text,Manual.Ustreditmenu);
      Write_Text(Scrolling_Text,Manual.Ustreditmenucontent);

      Write_Title_Small(Scrolling_Text,Manual.Ustrviewmenu);
      Write_Text(Scrolling_Text,Manual.Ustrviewmenucontent);

      Write_Title_Small(Scrolling_Text,Manual.Ustrtoolsmenu);
      Write_Text(Scrolling_Text,Manual.Ustrtoolsmenucontent);

      Write_Title_Small(Scrolling_Text,Manual.Ustrhelpmenu);
      Write_Text(Scrolling_Text,Manual.Ustrhelpmenucontent);

      Show_All(Scrolling_Text);
   end On_Manual_Activate;


   procedure On_Bibliography_References_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin
      Gtk_New(Scrolling_Text,Lb_Ref(Current_Language));
      Set_Usize(Scrolling_Text,435, 440);
      Write_Text(Scrolling_Text,References.Text);
      Show_All(Scrolling_Text);
   end On_Bibliography_References_Activate;


   procedure On_Pixmap_New_Clicked (
         Object : access Gtk_Toolbar_Record'Class ) is 
   begin
      Clear_Work_Space;
      Do_New;
   end On_Pixmap_New_Clicked;


   procedure On_Pixmap_Open_Clicked (
         Object : access Gtk_Toolbar_Record'Class ) is 
   begin
      Do_Open;
   end On_Pixmap_Open_Clicked;


   procedure On_Pixmap_Save_Clicked (
         Object : access Gtk_Toolbar_Record'Class ) is 
   begin
      Do_Save;
   end On_Pixmap_Save_Clicked;


   procedure On_Pixmap_Save_As_Clicked (
         Object : access Gtk_Toolbar_Record'Class ) is 
   begin
      Do_Save_As;
   end On_Pixmap_Save_As_Clicked;


   procedure On_Pixmap_Exit_Clicked (
         Object : access Gtk_Toolbar_Record'Class ) is 
   begin
      Do_Exit;
   end On_Pixmap_Exit_Clicked;


   procedure On_Pixmap_Clear_Work_Space_Clicked (
         Object : access Gtk_Toolbar_Record'Class ) is 
   begin
      Clear_Work_Space;
   end On_Pixmap_Clear_Work_Space_Clicked;


   procedure On_Pixmap2_Clicked (
         Object : access Gtk_Toolbar_Record'Class ) is 
   begin
      Show_Message_Box(Lb_No_Implem(Current_Language));
   end On_Pixmap2_Clicked;


   procedure On_Pixmap_Check_Scheduling_Clicked (
         Object : access Gtk_Toolbar_Record'Class ) is 
   begin
      Check_Scheduling;
   end On_Pixmap_Check_Scheduling_Clicked;


   procedure On_Pixmap_Draw_Scheduling_Clicked (
         Object : access Gtk_Toolbar_Record'Class ) is 

   begin
      Schedule_And_Check_Temporal_Constraints;
   end On_Pixmap_Draw_Scheduling_Clicked;


   procedure On_Delete_Task_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin
      if (Get_Number_Of_Elements(Sys.Tasks)=0)
            then
         Show_Message_Box(
            Lb_Define_Tasks_Before(Current_Language));
      else
         Gtk_New(Question, Lb_Delete_Task(Current_Language),
            Lb_Give_Task_Name(Current_Language),
            To_Unbounded_String(""),
            Delete_Task_Callback'access);
         Show_All(Question);
      end if;
   end On_Delete_Task_Activate;


   procedure On_Delete_Processor_Activate (
         Object : access Gtk_Menu_Item_Record'Class ) is 
   begin
      if (Get_Number_Of_Elements(Sys.Processors)=0)
            then
         Show_Message_Box(Lb_Define_Processors_Before(Current_Language));
      else
         Gtk_New(Question, Lb_Delete_Processor(Current_Language),
            Lb_Give_Processor_Name(Current_Language),
            To_Unbounded_String(""),
            Delete_Processor_Callback'access);
         Show_All(Question);
      end if;

   end On_Delete_Processor_Activate;


end Root_Window_Pkg.Callbacks;
