------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a real time simulator mainly designed for educational purpose. 
-- This program provides services to automaticly check temporal constraints 
-- of real time tasks.
--
-- Copyright (C) 2002   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the LIMI Team/EA 2215
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------



with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;

with Gtk.Box;
use Gtk.Box;
with Gtk.Menu_Bar;
use Gtk.Menu_Bar;
with Gtk.Menu_Item;
use Gtk.Menu_Item;
with Gtk.Menu;
use Gtk.Menu;
with Gtk.Toolbar;
use Gtk.Toolbar;
with Gtk.Scrolled_Window;
use Gtk.Scrolled_Window;
with Gtk.Text;
use Gtk.Text;
with Gtk.Window;
use Gtk.Window;
with Gtk.Pixmap;
use Gtk.Pixmap;
with Gdk.Font;
use Gdk.Font;
with Gtk.Paned;
use Gtk.Paned;
with Glib;
use Glib;
with Gdk.Color;
use Gdk.Color;


package Root_Window_Pkg is

   type Root_Window_Record is new Gtk_Window_Record with private; 
   type Root_Window_Access is access all Root_Window_Record'Class; 

   procedure Gtk_New (
         Root_Window :    out Root_Window_Access ); 
   procedure Initialize (
         Root_Window : access Root_Window_Record'Class ); 
   procedure Write_Text (
         Root_Window : in     Root_Window_Access; 
         Msg         :        Unbounded_String    ); 
   procedure Write_Title (
         Root_Window : in     Root_Window_Access; 
         Msg         :        Unbounded_String    ); 

   pragma Inline (Write_Text);
   pragma Inline (Write_Title);

   Root_Window : Root_Window_Access;  


private

   type Root_Window_Record is new Gtk_Window_Record with 
      record 
         Vbox            : Gtk_Vbox;  
         Vpaned          : Gtk_Vpaned;  
         Scrolledwindow1 : Gtk_Scrolled_Window;  
         Scrolledwindow2 : Gtk_Scrolled_Window;  
         Text            : Gtk_Text;  
         Font_R          : Gdk_Font;  
         Font_B          : Gdk_Font;  
         Red             : Gdk_Color;  
         Black           : Gdk_Color;  

         Separator1 : Gtk_Menu_Item;  
         Separator2 : Gtk_Menu_Item;  
         Separator3 : Gtk_Menu_Item;  

         Menubar                 : Gtk_Menu_Bar;  
         File                    : Gtk_Menu_Item;  
         File_Menu               : Gtk_Menu;  
         New_File                : Gtk_Menu_Item;  
         Open_File               : Gtk_Menu_Item;  
         Save_File               : Gtk_Menu_Item;  
         Save_As_File            : Gtk_Menu_Item;  
         Exit_Program            : Gtk_Menu_Item;  
         Edit                    : Gtk_Menu_Item;  
         Edit_Menu               : Gtk_Menu;  
         New_Task                : Gtk_Menu_Item;  
         New_Processor           : Gtk_Menu_Item;  
         Update_Task             : Gtk_Menu_Item;  
         Update_Processor        : Gtk_Menu_Item;  
         Delete_Task             : Gtk_Menu_Item;  
         Delete_Processor        : Gtk_Menu_Item;  
         Show                    : Gtk_Menu_Item;  
         Show_Menu               : Gtk_Menu;  
         Tasks_List              : Gtk_Menu_Item;  
         Processors_List         : Gtk_Menu_Item;  
         Tools                   : Gtk_Menu_Item;  
         Tools_Menu              : Gtk_Menu;  
         Draw_Scheduling_Graph   : Gtk_Menu_Item;  
         Check_Scheduling        : Gtk_Menu_Item;  
         Clear_Work_Space        : Gtk_Menu_Item;  
         Help                    : Gtk_Menu_Item;  
         Help_Menu               : Gtk_Menu;  
         About_Cheddar           : Gtk_Menu_Item;  
         Manual                  : Gtk_Menu_Item;  
         Bibliography_References : Gtk_Menu_Item;  

         Toolbar                 : Gtk_Toolbar;  
         Pixmap_Empty            : Gtk_Pixmap;  
         Pixmap_New              : Gtk_Pixmap;  
         Pixmap_Open             : Gtk_Pixmap;  
         Pixmap_Save             : Gtk_Pixmap;  
         Pixmap_Save_As          : Gtk_Pixmap;  
         Pixmap_Exit             : Gtk_Pixmap;  
         Pixmap_Clear_Work_Space : Gtk_Pixmap;  
         Pixmap2                 : Gtk_Pixmap;  
         Pixmap_Check_Scheduling : Gtk_Pixmap;  
         Pixmap_Draw_Scheduling  : Gtk_Pixmap;  
      end record; 

end Root_Window_Pkg;
