------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a real time simulator mainly designed for educational purpose. 
-- This program provides services to automaticly check temporal constraints 
-- of real time tasks.
--
-- Copyright (C) 2002   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the LIMI Team/EA 2215
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------



with Glib;
use Glib;

with Ihm_Tools;
use Ihm_Tools;

with Config;
use Config;

with Gtk.Radio_Menu_Item;
use Gtk.Radio_Menu_Item;

with Project;
use Project;
with Task_Set;
use Task_Set;
use Task_Set.Generic_Task_Set;
with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;
with Unbounded_Strings;
use Unbounded_Strings;

with Offsets;
use Offsets;

with Processors;
use Processors;
with Processor_Set;
use Processor_Set;
use Processor_Set.Generic_Processor_Set;

with Config;
use Config;
with Translate;
use Translate;


package body Tasks_Parameters_Pkg.Callbacks is




   procedure On_Ok_Pressed (
         Object : access Gtk_Button_Record'Class ) is 

      Offset      : Offset_Table;  
      A_Processor : Processor_Ptr;  
      Priority    : Integer;  
      Processor   : Unbounded_String;  
      Name        : Unbounded_String;  
      Capacity    : Integer;  
      Deadline    : Integer;  
      Period      : Integer;  
      Jitter      : Integer;  
      Start_Time  : Integer;  
      Ok          : Boolean;  
      Current     : Unbounded_String;  
      Cpu_Index   : Gint;  

   begin

      Name:=To_Unbounded_String(Get_Text(Tasks_Parameters.Entry_Name));

      if Name=""
            then
         Show_Message_Box(    Lb_Task_Name(Current_Language) &
            Lb_Mandatory(Current_Language) );
         return;
      end if;


      Cpu_Index:= Gint'Val (Gint(Get_Number_Of_Elements(Sys.Processors)) -
         Gint(Selected_Button (Tasks_Parameters.Clist_Processor_Menu)));
      Cpu_Index:=Cpu_Index-1;

      Get_Element_Number(Sys.Processors,A_Processor,Processors_Range(
            Cpu_Index));
      Processor:=A_Processor.Name;

      Ok:=True;
      Current:=To_Unbounded_String(Get_Text(Tasks_Parameters.
            Entry_Capacity));
      if(Current="")
            then
         Capacity:=0;
      else
         To_Integer(Current,Capacity,Ok);
         if not Ok
               then
            Show_Message_Box(Lb_Capacity(Current_Language) &
               Lb_Must_Be_Numeric(Current_Language) );
            return;
         end if;
      end if;


      Ok:=True;
      Current:=To_Unbounded_String(Get_Text(Tasks_Parameters.
            Entry_Deadline));
      if(Current="")
            then
         Deadline:=0;
      else
         To_Integer(Current,Deadline,Ok);
         if not Ok
               then
            Show_Message_Box(Lb_Deadline(Current_Language) &
               Lb_Must_Be_Numeric(Current_Language) );
            return;
         end if;
      end if;


      Ok:=True;
      Current:=To_Unbounded_String(Get_Text(Tasks_Parameters.Entry_Period));
      if(Current="")
            then
         Period:=0;
      else
         To_Integer(Current,Period,Ok);
         if not Ok
               then
            Show_Message_Box(Lb_Period(Current_Language) &
               Lb_Must_Be_Numeric(Current_Language) );
            return;
         end if;
      end if;


      Ok:=True;
      Current:=To_Unbounded_String(Get_Text(Tasks_Parameters.
            Entry_Start_Time));
      if(Current="")
            then
         Start_Time:=0;
      else
         To_Integer(Current,Start_Time,Ok);
         if not Ok
               then
            Show_Message_Box(Lb_Start_Time(Current_Language) &
               Lb_Must_Be_Numeric(Current_Language) );
            return;
         end if;
      end if;


      Ok:=True;
      Current:=To_Unbounded_String(Get_Text(Tasks_Parameters.
            Entry_Priority));
      if(Current="")
            then
         Priority:=Integer(Priority_Range'First);
      else
         To_Integer(Current,Priority,Ok);
         if not Ok
               then
            Show_Message_Box(Lb_Priority(Current_Language) &
               Lb_Must_Be_Numeric(Current_Language) );
            return;
         end if;
      end if;


      Ok:=True;
      Current:=To_Unbounded_String(Get_Text(Tasks_Parameters.Entry_Jitter));
      if(Current="")
            then
         Jitter:=0;
      else
         To_Integer(Current,Jitter,Ok);
         if not Ok
               then
            Show_Message_Box(Lb_Jitter(Current_Language) &
               Lb_Must_Be_Numeric(Current_Language) );
            return;
         end if;
      end if;


      --
      -- Offset information
      --
      Current:=To_Unbounded_String(Get_Text(Tasks_Parameters.
            Entry_Offset1_Val));
      if(Current="")
            then
         Offset(1).Value:=0;
      else
         To_Integer(Current,Offset(1).Value,Ok);
         if not Ok
               then
            Show_Message_Box(To_Unbounded_String("Offset") &
               Lb_Must_Be_Numeric(Current_Language) );
            return;
         end if;
      end if;
      Current:=To_Unbounded_String(Get_Text(Tasks_Parameters.
            Entry_Offset2_Val));
      if(Current="")
            then
         Offset(2).Value:=0;
      else
         To_Integer(Current,Offset(2).Value,Ok);
         if not Ok
               then
            Show_Message_Box(To_Unbounded_String("Offset") &
               Lb_Must_Be_Numeric(Current_Language) );
            return;
         end if;
      end if;
      Current:=To_Unbounded_String(Get_Text(Tasks_Parameters.
            Entry_Offset3_Val));
      if(Current="")
            then
         Offset(3).Value:=0;
      else
         To_Integer(Current,Offset(3).Value,Ok);
         if not Ok
               then
            Show_Message_Box(To_Unbounded_String("Offset") &
               Lb_Must_Be_Numeric(Current_Language) );
            return;
         end if;
      end if;
      Current:=To_Unbounded_String(Get_Text(Tasks_Parameters.
            Entry_Offset1_Act));
      if(Current="")
            then
         Offset(1).Activation:=0;
      else
         To_Integer(Current,Offset(1).Activation,Ok);
         if not Ok
               then
            Show_Message_Box(To_Unbounded_String("Offset") &
               Lb_Must_Be_Numeric(Current_Language) );
            return;
         end if;
      end if;
      Current:=To_Unbounded_String(Get_Text(Tasks_Parameters.
            Entry_Offset2_Act));
      if(Current="")
            then
         Offset(2).Activation:=0;
      else
         To_Integer(Current,Offset(2).Activation,Ok);
         if not Ok
               then
            Show_Message_Box(To_Unbounded_String("Offset") &
               Lb_Must_Be_Numeric(Current_Language) );
            return;
         end if;
      end if;
      Current:=To_Unbounded_String(Get_Text(Tasks_Parameters.
            Entry_Offset3_Act));
      if(Current="")
            then
         Offset(3).Activation:=0;
      else
         To_Integer(Current,Offset(3).Activation,Ok);
         if not Ok
               then
            Show_Message_Box(To_Unbounded_String("Offset") &
               Lb_Must_Be_Numeric(Current_Language) );
            return;
         end if;
      end if;



      if (Offset(1).Activation<0 or
            Offset(2).Activation<0 or
            Offset(3).Activation<0 )
            then
         Show_Message_Box(To_Unbounded_String("Offset/activation") &
            Lb_Must_Be(Current_Language)
            & To_Unbounded_String(" >= -1") );
         return;
      end if;


      if (Offset(1).Value<0 or Offset(2).Value<0 or Offset(3).Value<0)
            then
         Show_Message_Box(To_Unbounded_String("Offset/val.") &
            Lb_Must_Be(Current_Language)
            & To_Unbounded_String(" >= 0"));
         return;
      end if;



      if (Capacity<=0)
            then
         Show_Message_Box(Lb_Capacity(Current_Language) &
            Lb_Must_Be(Current_Language)
            & To_Unbounded_String(" > 0"));
         return;
      end if;

      if (Jitter<0)
            then
         Show_Message_Box(Lb_Jitter(Current_Language) &
            Lb_Must_Be(Current_Language)
            & To_Unbounded_String(" >= 0"));
         return;
      end if;

      if (Deadline<0)
            then
         Show_Message_Box(Lb_Deadline(Current_Language) &
            Lb_Must_Be(Current_Language)
            & To_Unbounded_String(" >= 0"));
         return;
      end if;

      if (Period<0)
            then
         Show_Message_Box(Lb_Period(Current_Language) &
            Lb_Must_Be(Current_Language)
            & To_Unbounded_String(" >= 0"));
         return;
      end if;

      if (Start_Time<0)
            then
         Show_Message_Box(Lb_Start_Time(Current_Language) &
            Lb_Must_Be(Current_Language)
            & To_Unbounded_String(" >= 0"));
         return;
      end if;

      if (  (Priority<Integer(Priority_Range'First))
            or (Priority>Integer(Priority_Range'Last)) )
            then
         Show_Message_Box(
            Lb_Invalid_Priority(Current_Language) );
         return;
      end if;


      Add_Task(Sys.Tasks,Name,Processor,Natural(Start_Time),
         Natural(Capacity),Natural(Period),Natural(Deadline),Natural(
            Jitter),0,Priority_Range(Priority),
         Offset);

      Is_New:=False;
      Is_Saved:=False;
      Hide(Tasks_Parameters);

   exception
      when Task_Already_Defined =>
         begin
            Show_Message_Box(Lb_Task_Name(Current_Language) &
               Lb_Already_Defined(Current_Language) );
            return;
         end;


   end On_Ok_Pressed;




   procedure On_Cancel_Pressed (
         Object : access Gtk_Button_Record'Class ) is 
   begin
      Hide(Tasks_Parameters);
   end On_Cancel_Pressed;

end Tasks_Parameters_Pkg.Callbacks;
