------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a real time simulator mainly designed for educational purpose. 
-- This program provides services to automaticly check temporal constraints 
-- of real time tasks.
--
-- Copyright (C) 2002   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the LIMI Team/EA 2215
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------



with Glib;
use Glib;
with Gtk;
use Gtk;
with Gdk.Types;
use Gdk.Types;
with Gtk.Enums;
use Gtk.Enums;
with Callbacks_Cheddar;
use Callbacks_Cheddar;
with Tasks_Parameters_Pkg.Callbacks;
use Tasks_Parameters_Pkg.Callbacks;
with Processors;
use Processors;
with Project;
use Project;
with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;
with  Unbounded_Strings;
use  Unbounded_Strings;
with Ihm_Tools;
use Ihm_Tools;

with Config;
use Config;

with Translate;
use Translate;


package body Tasks_Parameters_Pkg is

   procedure Gtk_New (
         Tasks_Parameters :    out Tasks_Parameters_Access ) is 
   begin
      Tasks_Parameters := new Tasks_Parameters_Record;
      Tasks_Parameters_Pkg.Initialize (Tasks_Parameters);
   end Gtk_New;

   procedure Initialize (
         Tasks_Parameters : access Tasks_Parameters_Record'Class ) is 

      A_Processor : Processor_Ptr;  
      My_Iterator : Iterator;  
      Items       : Strings_Table;  
      Index       : Strings_Table_Range := Strings_Table_Range'First;  

   begin
      Gtk.Window.Initialize (Tasks_Parameters, Window_Toplevel);
      Set_Title (Tasks_Parameters, To_String(Lb_Add_Task(Current_Language)));
      Set_Policy (Tasks_Parameters, True, True, True);
      Set_Position (Tasks_Parameters, Win_Pos_Center_Always);
      Set_Modal (Tasks_Parameters, True);
      Set_Extension_Events (Tasks_Parameters, Extension_Events_All);
      Set_Flags (Tasks_Parameters, Can_Default);
      Grab_Focus (Tasks_Parameters);

      Gtk_New (Tasks_Parameters.Vbox, 8, 2, True);
      Set_Border_Width (Tasks_Parameters.Vbox, 14);
      Set_Row_Spacings (Tasks_Parameters.Vbox, 8);
      Set_Col_Spacings (Tasks_Parameters.Vbox, 0);
      Add (Tasks_Parameters, Tasks_Parameters.Vbox);

      Gtk_New (Tasks_Parameters.Name, To_String(Lb_Task_Name(
               Current_Language)));
      Set_Alignment (Tasks_Parameters.Name, 0.5, 0.5);
      Set_Padding (Tasks_Parameters.Name, 0, 0);
      Set_Justify (Tasks_Parameters.Name, Justify_Left);
      Set_Line_Wrap (Tasks_Parameters.Name, False);
      Attach (Tasks_Parameters.Vbox, Tasks_Parameters.Name, 0, 1, 0, 1,
         0, 0,
         0, 0);

      Gtk_New (Tasks_Parameters.Processor, To_String(Lb_Processor_Name(
               Current_Language) & Lb_Colon));
      Set_Alignment (Tasks_Parameters.Processor, 0.5, 0.5);
      Set_Padding (Tasks_Parameters.Processor, 0, 0);
      Set_Justify (Tasks_Parameters.Processor, Justify_Fill);
      Set_Line_Wrap (Tasks_Parameters.Processor, True);
      Attach (Tasks_Parameters.Vbox, Tasks_Parameters.Processor, 0, 1, 1,
         2,
         0, 0,
         0, 0);

      Gtk_New (Tasks_Parameters.Capacity, To_String(Lb_Capacity(
               Current_Language) & Lb_Colon));
      Set_Alignment (Tasks_Parameters.Capacity, 0.5, 0.5);
      Set_Padding (Tasks_Parameters.Capacity, 0, 0);
      Set_Justify (Tasks_Parameters.Capacity, Justify_Left);
      Set_Line_Wrap (Tasks_Parameters.Capacity, False);
      Attach (Tasks_Parameters.Vbox, Tasks_Parameters.Capacity, 0, 1, 2, 3,
         0, 0,
         0, 0);

      Gtk_New (Tasks_Parameters.Jitter, To_String(Lb_Jitter(
               Current_Language) & Lb_Colon));
      Set_Alignment (Tasks_Parameters.Jitter, 0.5, 0.5);
      Set_Padding (Tasks_Parameters.Jitter, 0, 0);
      Set_Justify (Tasks_Parameters.Jitter, Justify_Left);
      Set_Line_Wrap (Tasks_Parameters.Jitter, False);
      Attach (Tasks_Parameters.Vbox, Tasks_Parameters.Jitter, 0, 1, 3, 4,
         0, 0,
         0, 0);

      Gtk_New (Tasks_Parameters.Deadline, To_String(Lb_Deadline(
               Current_Language) & Lb_Colon));
      Set_Alignment (Tasks_Parameters.Deadline, 0.5, 0.5);
      Set_Padding (Tasks_Parameters.Deadline, 0, 0);
      Set_Justify (Tasks_Parameters.Deadline, Justify_Left);
      Set_Line_Wrap (Tasks_Parameters.Deadline, False);
      Attach (Tasks_Parameters.Vbox, Tasks_Parameters.Deadline, 0, 1, 4, 5,
         0, 0,
         0, 0);

      Gtk_New (Tasks_Parameters.Period, To_String(Lb_Period(
               Current_Language) & Lb_Colon));
      Set_Alignment (Tasks_Parameters.Period, 0.5, 0.5);
      Set_Padding (Tasks_Parameters.Period, 0, 0);
      Set_Justify (Tasks_Parameters.Period, Justify_Center);
      Set_Line_Wrap (Tasks_Parameters.Period, False);
      Attach (Tasks_Parameters.Vbox, Tasks_Parameters.Period, 0, 1, 5, 6,
         0, 0,
         0, 0);

      Gtk_New (Tasks_Parameters.Start_Time, To_String(Lb_Start_Time(
               Current_Language) & Lb_Colon));
      Set_Alignment (Tasks_Parameters.Start_Time, 0.5, 0.5);
      Set_Padding (Tasks_Parameters.Start_Time, 0, 0);
      Set_Justify (Tasks_Parameters.Start_Time, Justify_Center);
      Set_Line_Wrap (Tasks_Parameters.Start_Time, False);
      Attach (Tasks_Parameters.Vbox, Tasks_Parameters.Start_Time, 0, 1, 6,
         7,
         0, 0,
         0, 0);

      Gtk_New (Tasks_Parameters.Priority, To_String(Lb_Priority(
               Current_Language) & Lb_Colon));
      Set_Alignment (Tasks_Parameters.Priority, 0.5, 0.5);
      Set_Padding (Tasks_Parameters.Priority, 0, 0);
      Set_Justify (Tasks_Parameters.Priority, Justify_Center);
      Set_Line_Wrap (Tasks_Parameters.Priority, False);
      Attach (Tasks_Parameters.Vbox, Tasks_Parameters.Priority, 0, 1, 7, 8,
         0, 0,
         0, 0);


      Gtk_New_Hseparator (Tasks_Parameters.Hseparator1);
      Attach (Tasks_Parameters.Vbox, Tasks_Parameters.Hseparator1, 1, 2, 8,
         9,
         Fill, Expand or Fill,
         0, 0);

      Gtk_New_Hseparator (Tasks_Parameters.Hseparator2);
      Attach (Tasks_Parameters.Vbox, Tasks_Parameters.Hseparator2, 0, 1, 8,
         9,
         Fill, Fill,
         0, 0);

      Gtk_New (Tasks_Parameters.Priority, "Offsets");
      Set_Alignment (Tasks_Parameters.Priority, 0.5, 0.5);
      Set_Padding (Tasks_Parameters.Priority, 0, 0);
      Set_Justify (Tasks_Parameters.Priority, Justify_Center);
      Set_Line_Wrap (Tasks_Parameters.Priority, False);
      Attach (Tasks_Parameters.Vbox, Tasks_Parameters.Priority, 0, 1,  9,
         10,
         0, 0,
         0, 0);

      Gtk_New (Tasks_Parameters.Priority, "Activations");
      Set_Alignment (Tasks_Parameters.Priority, 0.5, 0.5);
      Set_Padding (Tasks_Parameters.Priority, 0, 0);
      Set_Justify (Tasks_Parameters.Priority, Justify_Center);
      Set_Line_Wrap (Tasks_Parameters.Priority, False);
      Attach (Tasks_Parameters.Vbox, Tasks_Parameters.Priority, 0, 1,   10,
         11,
         0, 0,
         0, 0);

      Gtk_New (Tasks_Parameters.Priority, To_String(Lb_Values(
               Current_Language) & Lb_Colon));
      Set_Alignment (Tasks_Parameters.Priority, 0.5, 0.5);
      Set_Padding (Tasks_Parameters.Priority, 0, 0);
      Set_Justify (Tasks_Parameters.Priority, Justify_Center);
      Set_Line_Wrap (Tasks_Parameters.Priority, False);
      Attach (Tasks_Parameters.Vbox, Tasks_Parameters.Priority, 1, 2, 10,
         11,
         0, 0,
         0, 0);

      Gtk_New (Tasks_Parameters.Entry_Name);
      Set_Editable (Tasks_Parameters.Entry_Name, True);
      Set_Max_Length (Tasks_Parameters.Entry_Name, 0);
      Set_Text (Tasks_Parameters.Entry_Name, "");
      Set_Visibility (Tasks_Parameters.Entry_Name, True);
      Attach (Tasks_Parameters.Vbox, Tasks_Parameters.Entry_Name, 1, 2, 0,
         1,
         Expand or Fill, 0,
         0, 0);


      Initialize(Items);
      Index:=0;
      Reset_Iterator(My_Iterator);

      loop
         Current_Element(Sys.Processors, A_Processor, My_Iterator);
         Items(Index):=A_Processor.Name;
         Index:=Index+1;

         exit when Is_Last_Element(Sys.Processors, My_Iterator) = True;

         Next_Element(Sys.Processors, My_Iterator);
      end loop;


      Tasks_Parameters.Clist_Processor_Menu:= Widget_Slist.Null_List;
      Build_Option_Menu (Tasks_Parameters.Processor_Menu,
         Tasks_Parameters.Clist_Processor_Menu, Items, -1, null);

      Attach (Tasks_Parameters.Vbox, Tasks_Parameters.Processor_Menu, 1, 2,
         1, 2,
         Expand or Fill, 0,
         0, 0);


      Gtk_New (Tasks_Parameters.Entry_Capacity);
      Set_Editable (Tasks_Parameters.Entry_Capacity, True);
      Set_Max_Length (Tasks_Parameters.Entry_Capacity, 0);
      Set_Text (Tasks_Parameters.Entry_Capacity, "");
      Set_Visibility (Tasks_Parameters.Entry_Capacity, True);
      Attach (Tasks_Parameters.Vbox, Tasks_Parameters.Entry_Capacity, 1, 2,
         2, 3,
         Expand or Fill, 0,
         0, 0);

      Gtk_New (Tasks_Parameters.Entry_Jitter);
      Set_Editable (Tasks_Parameters.Entry_Jitter, True);
      Set_Max_Length (Tasks_Parameters.Entry_Jitter, 0);
      Set_Text (Tasks_Parameters.Entry_Jitter, "");
      Set_Visibility (Tasks_Parameters.Entry_Jitter, True);
      Attach (Tasks_Parameters.Vbox, Tasks_Parameters.Entry_Jitter, 1, 2,
         3, 4,
         Expand or Fill, 0,
         0, 0);

      Gtk_New (Tasks_Parameters.Entry_Deadline);
      Set_Editable (Tasks_Parameters.Entry_Deadline, True);
      Set_Max_Length (Tasks_Parameters.Entry_Deadline, 0);
      Set_Text (Tasks_Parameters.Entry_Deadline, "");
      Set_Visibility (Tasks_Parameters.Entry_Deadline, True);
      Attach (Tasks_Parameters.Vbox, Tasks_Parameters.Entry_Deadline, 1, 2,
         4, 5,
         Expand or Fill, 0,
         0, 0);

      Gtk_New (Tasks_Parameters.Entry_Period);
      Set_Editable (Tasks_Parameters.Entry_Period, True);
      Set_Max_Length (Tasks_Parameters.Entry_Period, 0);
      Set_Text (Tasks_Parameters.Entry_Period, "");
      Set_Visibility (Tasks_Parameters.Entry_Period, True);
      Attach (Tasks_Parameters.Vbox, Tasks_Parameters.Entry_Period, 1, 2,
         5, 6,
         Expand or Fill, 0,
         0, 0);

      Gtk_New (Tasks_Parameters.Entry_Start_Time);
      Set_Editable (Tasks_Parameters.Entry_Start_Time, True);
      Set_Max_Length (Tasks_Parameters.Entry_Start_Time, 0);
      Set_Text (Tasks_Parameters.Entry_Start_Time, "");
      Set_Visibility (Tasks_Parameters.Entry_Start_Time, True);
      Attach (Tasks_Parameters.Vbox, Tasks_Parameters.Entry_Start_Time, 1,
         2, 6, 7,
         Expand or Fill, 0,
         0, 0);

      Gtk_New (Tasks_Parameters.Entry_Priority);
      Set_Editable (Tasks_Parameters.Entry_Priority, True);
      Set_Max_Length (Tasks_Parameters.Entry_Priority, 0);
      Set_Text (Tasks_Parameters.Entry_Priority, "");
      Set_Visibility (Tasks_Parameters.Entry_Priority, True);
      Attach (Tasks_Parameters.Vbox, Tasks_Parameters.Entry_Priority, 1, 2,
         7, 8,
         Expand or Fill, 0,
         0, 0);

      Gtk_New (Tasks_Parameters.Entry_Offset1_Act);
      Set_Editable (Tasks_Parameters.Entry_Offset1_Act, True);
      Set_Max_Length (Tasks_Parameters.Entry_Offset1_Act, 0);
      Set_Text (Tasks_Parameters.Entry_Offset1_Act, "");
      Set_Visibility (Tasks_Parameters.Entry_Offset1_Act, True);
      Attach (Tasks_Parameters.Vbox, Tasks_Parameters.Entry_Offset1_Act, 0,
         1, 11, 12,
         Expand or Fill, 0,
         0, 0);

      Gtk_New (Tasks_Parameters.Entry_Offset1_Val);
      Set_Editable (Tasks_Parameters.Entry_Offset1_Val, True);
      Set_Max_Length (Tasks_Parameters.Entry_Offset1_Val, 0);
      Set_Text (Tasks_Parameters.Entry_Offset1_Val, "");
      Set_Visibility (Tasks_Parameters.Entry_Offset1_Val, True);
      Attach (Tasks_Parameters.Vbox, Tasks_Parameters.Entry_Offset1_Val, 1,
         2,  11, 12,
         Expand or Fill, 0,
         0, 0);

      Gtk_New (Tasks_Parameters.Entry_Offset2_Act);
      Set_Editable (Tasks_Parameters.Entry_Offset2_Act, True);
      Set_Max_Length (Tasks_Parameters.Entry_Offset2_Act, 0);
      Set_Text (Tasks_Parameters.Entry_Offset2_Act, "");
      Set_Visibility (Tasks_Parameters.Entry_Offset2_Act, True);
      Attach (Tasks_Parameters.Vbox, Tasks_Parameters.Entry_Offset2_Act, 0,
         1, 12, 13,
         Expand or Fill, 0,
         0, 0);

      Gtk_New (Tasks_Parameters.Entry_Offset2_Val);
      Set_Editable (Tasks_Parameters.Entry_Offset2_Val, True);
      Set_Max_Length (Tasks_Parameters.Entry_Offset2_Val, 0);
      Set_Text (Tasks_Parameters.Entry_Offset2_Val, "");
      Set_Visibility (Tasks_Parameters.Entry_Offset2_Val, True);
      Attach (Tasks_Parameters.Vbox, Tasks_Parameters.Entry_Offset2_Val, 1,
         2,  12, 13,
         Expand or Fill, 0,
         0, 0);
      Gtk_New (Tasks_Parameters.Entry_Offset3_Act);
      Set_Editable (Tasks_Parameters.Entry_Offset3_Act, True);
      Set_Max_Length (Tasks_Parameters.Entry_Offset3_Act, 0);
      Set_Text (Tasks_Parameters.Entry_Offset3_Act, "");
      Set_Visibility (Tasks_Parameters.Entry_Offset3_Act, True);
      Attach (Tasks_Parameters.Vbox, Tasks_Parameters.Entry_Offset3_Act, 0,
         1, 13, 14,
         Expand or Fill, 0,
         0, 0);

      Gtk_New (Tasks_Parameters.Entry_Offset3_Val);
      Set_Editable (Tasks_Parameters.Entry_Offset3_Val, True);
      Set_Max_Length (Tasks_Parameters.Entry_Offset3_Val, 0);
      Set_Text (Tasks_Parameters.Entry_Offset3_Val, "");
      Set_Visibility (Tasks_Parameters.Entry_Offset3_Val, True);
      Attach (Tasks_Parameters.Vbox, Tasks_Parameters.Entry_Offset3_Val, 1,
         2,  13, 14,
         Expand or Fill, 0,
         0, 0);


      Gtk_New_Hseparator (Tasks_Parameters.Hseparator1);
      Attach (Tasks_Parameters.Vbox, Tasks_Parameters.Hseparator1, 1, 2,
         14, 15,
         Fill, Expand or Fill,
         0, 0);

      Gtk_New_Hseparator (Tasks_Parameters.Hseparator2);
      Attach (Tasks_Parameters.Vbox, Tasks_Parameters.Hseparator2, 0, 1,
         14, 15,
         Fill, Fill,
         0, 0);

      Gtk_New_Hbox (Tasks_Parameters.Hbox, True, 0);
      Set_Border_Width (Tasks_Parameters.Hbox, 6);
      Attach (Tasks_Parameters.Vbox, Tasks_Parameters.Hbox, 0, 2, 15, 16,
         Fill, Expand or Fill,
         0, 0);

      Gtk_New (Tasks_Parameters.Ok_Button, "OK");
      Pack_Start (Tasks_Parameters.Hbox, Tasks_Parameters.Ok_Button, True,
         True, 0);
      Button_Callback.Connect
         (Tasks_Parameters.Ok_Button, "clicked",
         Button_Callback.To_Marshaller (On_Ok_Pressed'access));

      Gtk_New_Vseparator (Tasks_Parameters.Vseparator);
      Set_Sensitive (Tasks_Parameters.Vseparator, False);
      Pack_Start (Tasks_Parameters.Hbox, Tasks_Parameters.Vseparator,  True,
         True, 0);

      Gtk_New (Tasks_Parameters.Cancel_Button, To_String(Lb_Cancel(
               Current_Language)));
      Pack_Start (Tasks_Parameters.Hbox, Tasks_Parameters.Cancel_Button,
         True, True, 0);
      Button_Callback.Connect
         (Tasks_Parameters.Cancel_Button, "clicked",
         Button_Callback.To_Marshaller (On_Cancel_Pressed'access));


   end Initialize;

end Tasks_Parameters_Pkg;




