------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a real time simulator mainly designed for educational purpose. 
-- This program provides services to automaticly check temporal constraints 
-- of real time tasks.
--
-- Copyright (C) 2002   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the LIMI Team/EA 2215
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------



with Glib;
use Glib;

with Ihm_Tools;
use Ihm_Tools;

with Config;
use Config;

with Offsets;
use Offsets;
with Unbounded_Strings;
use Unbounded_Strings;

with Project;
use Project;
with Task_Set;
use Task_Set;
use Task_Set.Generic_Task_Set;
with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;

with Processors;
use Processors;
with Processor_Set;
use Processor_Set;
use Processor_Set.Generic_Processor_Set;

with Translate;
use Translate;

with Gtk.Radio_Menu_Item;
use Gtk.Radio_Menu_Item;


package body Update_Tasks_Set_Pkg.Callbacks is



   procedure On_Ok_Pressed (
         Object : access Gtk_Button_Record'Class ) is 

      Nb_Tasks : Tasks_Range := Get_Number_Of_Elements (Sys.Tasks);  
      type A_Task is 
         record 
            Priority   : Integer;  
            Processor  : Unbounded_String;  
            Name       : Unbounded_String;  
            Capacity   : Integer;  
            Deadline   : Integer;  
            Period     : Integer;  
            Jitter     : Integer;  
            Start_Time : Integer;  
            Offset     : Offset_Table;  
         end record; 
      type Tasks_List is array (Tasks_Range) of A_Task; 

      List        : Tasks_List;  
      K,  
      L           : Tasks_Range;  
      A_Processor : Processor_Ptr;  
      Cpu_Index   : Gint;  

      Ok      : Boolean;  
      Current : Unbounded_String;  

   begin

      for I in 0..Nb_Tasks-1 loop


         L:=I+1;

         List(I).Name:=To_Unbounded_String(Get_Text(Update_Tasks_Set.
               Entries(I).Name));

         Cpu_Index:= Gint'Val (Gint(Get_Number_Of_Elements(Sys.Processors)) -
            Gint(Selected_Button (Update_Tasks_Set.Entries(I).
                  Clist_Processor_Menu)));
         Cpu_Index:=Cpu_Index-1;

         Get_Element_Number(Sys.Processors,A_Processor,Processors_Range(
               Cpu_Index));
         List(I).Processor:=A_Processor.Name;



         if List(I).Name=""
               then
            Show_Message_Box(Lb_Task_Name(Current_Language) &
               Lb_Mandatory(Current_Language) &
               To_Unbounded_String(" (ligne " & L'Img & ")") );
            return;
         end if;

         if List(I).Processor=""
               then
            Show_Message_Box(Lb_Processor_Name(Current_Language) &
               Lb_Mandatory(Current_Language) &
               To_Unbounded_String(" (ligne " & L'Img & ")") );
            return;
         end if;




         Ok:=True;
         Current:=To_Unbounded_String(Get_Text(Update_Tasks_Set.Entries(I).
               Capacity));
         if(Current="")
               then
            List(I).Capacity:=0;
         else
            To_Integer(Current,List(I).Capacity,Ok);
            if not Ok
                  then
               Show_Message_Box(Lb_Capacity(Current_Language) &
                  Lb_Must_Be_Numeric(Current_Language) &
                  To_Unbounded_String(" (ligne " & L'Img & ")") );
               return;
            end if;
         end if;


         Ok:=True;
         Current:=To_Unbounded_String(Get_Text(Update_Tasks_Set.Entries(I).
               Deadline));
         if(Current="")
               then
            List(I).Deadline:=0;
         else
            To_Integer(Current,List(I).Deadline,Ok);
            if not Ok
                  then
               Show_Message_Box(Lb_Deadline(Current_Language) &
                  Lb_Must_Be_Numeric(Current_Language) &
                  To_Unbounded_String(" (ligne " & L'Img & ")") );
               return;
            end if;
         end if;


         Ok:=True;
         Current:=To_Unbounded_String(Get_Text(Update_Tasks_Set.Entries(I).
               Period));
         if(Current="")
               then
            List(I).Period:=0;
         else
            To_Integer(Current,List(I).Period,Ok);
            if not Ok
                  then
               Show_Message_Box(Lb_Period(Current_Language) &
                  Lb_Must_Be_Numeric(Current_Language) &
                  To_Unbounded_String(" (ligne " & L'Img & ")") );
               return;
            end if;
         end if;


         Ok:=True;
         Current:=To_Unbounded_String(Get_Text(Update_Tasks_Set.Entries(I).
               Start_Time));
         if(Current="")
               then
            List(I).Start_Time:=0;
         else
            To_Integer(Current,List(I).Start_Time,Ok);
            if not Ok
                  then
               Show_Message_Box(Lb_Start_Time(Current_Language) &
                  Lb_Must_Be_Numeric(Current_Language) &
                  To_Unbounded_String(" (ligne " & L'Img & ")") );
               return;
            end if;
         end if;


         Ok:=True;
         Current:=To_Unbounded_String(Get_Text(Update_Tasks_Set.Entries(I).
               Priority));
         if(Current="")
               then
            List(I).Priority:=Natural(Priority_Range'First);
         else
            To_Integer(Current,List(I).Priority,Ok);
            if not Ok
                  then
               Show_Message_Box(Lb_Priority(Current_Language) &
                  Lb_Must_Be_Numeric(Current_Language) &
                  To_Unbounded_String(" (ligne " & L'Img & ")") );
               return;
            end if;
         end if;


         Ok:=True;
         Current:=To_Unbounded_String(Get_Text(Update_Tasks_Set.Entries(I).
               Jitter));
         if(Current="")
               then
            List(I).Jitter:=0;
         else
            To_Integer(Current,List(I).Jitter,Ok);
            if not Ok
                  then
               Show_Message_Box(Lb_Jitter(Current_Language) &
                  Lb_Must_Be_Numeric(Current_Language) &
                  To_Unbounded_String(" (ligne " & L'Img & ")") );
               return;
            end if;
         end if;


         --
         -- Offset information
         --
         Current:=To_Unbounded_String(Get_Text(Update_Tasks_Set.Entries(I).
               Offset_Val1));
         if(Current="")
               then
            List(I).Offset(1).Value:=0;
         else
            To_Integer(Current,List(I).Offset(1).Value,Ok);
            if not Ok
                  then
               Show_Message_Box(To_Unbounded_String("Offset") &
                  Lb_Must_Be_Numeric(Current_Language) &
                  To_Unbounded_String(" (ligne " & L'Img & ")") );
               return;
            end if;
         end if;
         Current:=To_Unbounded_String(Get_Text(Update_Tasks_Set.Entries(I).
               Offset_Val2));
         if(Current="")
               then
            List(I).Offset(2).Value:=0;
         else
            To_Integer(Current,List(I).Offset(2).Value,Ok);
            if not Ok
                  then
               Show_Message_Box(To_Unbounded_String("Offset") &
                  Lb_Must_Be_Numeric(Current_Language) &
                  To_Unbounded_String(" (ligne " & L'Img & ")") );
               return;
            end if;
         end if;
         Current:=To_Unbounded_String(Get_Text(Update_Tasks_Set.Entries(I).
               Offset_Val3));
         if(Current="")
               then
            List(I).Offset(3).Value:=0;
         else
            To_Integer(Current,List(I).Offset(3).Value,Ok);
            if not Ok
                  then
               Show_Message_Box(To_Unbounded_String("Offset") &
                  Lb_Must_Be_Numeric(Current_Language) &
                  To_Unbounded_String(" (ligne " & L'Img & ")") );
               return;
            end if;
         end if;
         Current:=To_Unbounded_String(Get_Text(Update_Tasks_Set.Entries(I).
               Offset_Act1));
         if(Current="")
               then
            List(I).Offset(1).Activation:=0;
         else
            To_Integer(Current,List(I).Offset(1).Activation,Ok);
            if not Ok
                  then
               Show_Message_Box(To_Unbounded_String("Offset") &
                  Lb_Must_Be_Numeric(Current_Language) &
                  To_Unbounded_String(" (ligne " & L'Img & ")") );
               return;
            end if;
         end if;
         Current:=To_Unbounded_String(Get_Text(Update_Tasks_Set.Entries(I).
               Offset_Act2));
         if(Current="")
               then
            List(I).Offset(2).Activation:=0;
         else
            To_Integer(Current,List(I).Offset(2).Activation,Ok);
            if not Ok
                  then
               Show_Message_Box(To_Unbounded_String("Offset") &
                  Lb_Must_Be_Numeric(Current_Language) &
                  To_Unbounded_String(" (ligne " & L'Img & ")") );
               return;
            end if;
         end if;
         Current:=To_Unbounded_String(Get_Text(Update_Tasks_Set.Entries(I).
               Offset_Act3));
         if(Current="")
               then
            List(I).Offset(3).Activation:=0;
         else
            To_Integer(Current,List(I).Offset(3).Activation,Ok);
            if not Ok
                  then
               Show_Message_Box(To_Unbounded_String("Offset") &
                  Lb_Must_Be_Numeric(Current_Language) &
                  To_Unbounded_String(" (ligne " & L'Img & ")") );
               return;
            end if;
         end if;



         if (List(I).Offset(1).Activation<-0 or List(I).Offset(2).
               Activation<0 or List(I).Offset(3).
               Activation<0 )
               then
            Show_Message_Box(To_Unbounded_String("Offset/activation") &
               Lb_Must_Be(Current_Language)
               & To_Unbounded_String(" >= 0") &
               To_Unbounded_String(" (ligne " & L'Img & ")") );
            return;
         end if;


         if (List(I).Offset(1).Value<0 or List(I).Offset(2).Value<0 or
               List(I).Offset(3).Value<0)
               then
            Show_Message_Box(To_Unbounded_String("Offset/val.") &
               Lb_Must_Be(Current_Language)
               & To_Unbounded_String(" >= 0") &
               To_Unbounded_String(" (ligne " & L'Img & ")") );
            return;
         end if;



         if (List(I).Capacity<=0)
               then
            Show_Message_Box(Lb_Capacity(Current_Language) &
               Lb_Must_Be(Current_Language)
               & To_Unbounded_String(" > 0") &
               To_Unbounded_String(" (ligne " & L'Img & ")") );
            return;
         end if;

         if (List(I).Jitter<0)
               then
            Show_Message_Box(Lb_Jitter(Current_Language) &
               Lb_Must_Be(Current_Language)
               & To_Unbounded_String(" >= 0") &
               To_Unbounded_String(" (ligne " & L'Img & ")") );
            return;
         end if;

         if (List(I).Deadline<0)
               then
            Show_Message_Box(Lb_Deadline(Current_Language) &
               Lb_Must_Be(Current_Language)
               & To_Unbounded_String(" >= 0") &
               To_Unbounded_String(" (ligne " & L'Img & ")") );
            return;
         end if;

         if (List(I).Period<0)
               then
            Show_Message_Box(Lb_Period(Current_Language) &
               Lb_Must_Be(Current_Language)
               & To_Unbounded_String(" >= 0") &
               To_Unbounded_String(" (ligne " & L'Img & ")") );
            return;
         end if;

         if (List(I).Start_Time<0)
               then
            Show_Message_Box(Lb_Start_Time(Current_Language) &
               Lb_Must_Be(Current_Language)
               & To_Unbounded_String(" >= 0") &
               To_Unbounded_String(" (ligne " & L'Img & ")") );
            return;
         end if;

         if (  (List(I).Priority<Natural(Priority_Range'First))
               or (List(I).Priority>Natural(Priority_Range'Last)) )
               then
            Show_Message_Box(
               Lb_Invalid_Priority(Current_Language) &
               To_Unbounded_String(" (ligne " & L'Img & ")") );
            return;
         end if;


      end loop;



      for I in 0..Nb_Tasks-1 loop
         for J in 0..Nb_Tasks-1 loop
            if ( (I /= J)
                  and (List(I).Name=List(J).Name) )
                  then
               L:=I+1;
               K:=J+1;
               Show_Message_Box(
                  Lb_Task_Name(Current_Language) &
                  Lb_Already_Defined(Current_Language) &
                  To_Unbounded_String(" (lignes " & L'Img & ", " & K'Img &
                     ")") );
               return;
            end if;
         end loop;
      end loop;

      Reset(Sys.Tasks);

      for I in 0..Nb_Tasks-1 loop
         Add_Task(Sys.Tasks,List(I).Name,List(I).Processor,Natural(List(I).
               Start_Time),
            Natural(List(I).Capacity),Natural(List(I).Period),Natural(
               List(I).Deadline),Natural(List(I).
               Jitter),
            0,Priority_Range(List(I).Priority),List(I).Offset);
      end loop;

      Is_Saved:=False;
      Hide(Update_Tasks_Set);

   end On_Ok_Pressed;


   procedure On_Cancel_Pressed (
         Object : access Gtk_Button_Record'Class ) is 

   begin

      Hide(Update_Tasks_Set);

   end On_Cancel_Pressed;



end Update_Tasks_Set_Pkg.Callbacks;
