------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a real time simulator mainly designed for educational purpose. 
-- This program provides services to automaticly check temporal constraints 
-- of real time tasks.
--
-- Copyright (C) 2002   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the LIMI Team/EA 2215
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------



with Glib;
use Glib;
with Gtk;
use Gtk;
with Gdk.Types;
use Gdk.Types;
with Gtk.Widget;
use Gtk.Widget;
with Gtk.Enums;
use Gtk.Enums;

with Ihm_Tools;
use Ihm_Tools;
with Callbacks_Cheddar;
use Callbacks_Cheddar;
with Update_Tasks_Set_Pkg.Callbacks;
use Update_Tasks_Set_Pkg.Callbacks;


with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;
with Processors;
use Processors;
with Unbounded_Strings;
use  Unbounded_Strings;
with Project;
use Project;
with Tasks;
use Tasks;
with Task_Set;
use Task_Set;
use Task_Set.Generic_Task_Set;
with Config;
use Config;
with Translate;
use Translate;


package body Update_Tasks_Set_Pkg is

   procedure Gtk_New (
         Update_Tasks_Set :    out Update_Tasks_Set_Access ) is 
   begin
      Update_Tasks_Set := new Update_Tasks_Set_Record;
      Update_Tasks_Set_Pkg.Initialize (Update_Tasks_Set);
   end Gtk_New;

   procedure Initialize (
         Update_Tasks_Set : access Update_Tasks_Set_Record'Class ) is 


      A_Task      : Task_Ptr;  
      Titerator   : Tasks_Iterator;  
      Ligne       : Guint               := 1;  
      Heigth      : Gint;  
      A_Processor : Processor_Ptr;  
      Piterator   : Processors_Iterator;  
      Items       : Strings_Table;  
      Index       : Strings_Table_Range := Strings_Table_Range'First;  
      Current     : Gint;  



   begin
      Gtk.Window.Initialize (Update_Tasks_Set, Window_Toplevel);
      Set_Title (Update_Tasks_Set, To_String(Lb_Update_Task(
               Current_Language)));
      Set_Policy (Update_Tasks_Set, True, True, True);
      Heigth:=Gint(3+Get_Number_Of_Elements(Sys.Tasks))*50;
      if Heigth > 400
            then
         Heigth:=400;
      end if;
      Set_Default_Size (Update_Tasks_Set, 600, Heigth);
      Set_Position (Update_Tasks_Set, Win_Pos_Center_Always);
      Set_Modal (Update_Tasks_Set, True);
      Set_Extension_Events (Update_Tasks_Set, Extension_Events_All);
      Set_Flags (Update_Tasks_Set, Can_Default);
      Grab_Focus (Update_Tasks_Set);


      Gtk_New (Update_Tasks_Set.Scroll);
      Set_Border_Width (Update_Tasks_Set.Scroll, Border_Width => 10);
      Set_Policy (Update_Tasks_Set.Scroll,
         H_Scrollbar_Policy => Policy_Automatic,
         V_Scrollbar_Policy => Policy_Automatic);
      Add (Update_Tasks_Set, Update_Tasks_Set.Scroll);


      Gtk_New (Update_Tasks_Set.Table, Guint(3+Get_Number_Of_Elements(
               Sys.Tasks)), 7, True);
      Set_Row_Spacings (Update_Tasks_Set.Table, 8);
      Set_Col_Spacings (Update_Tasks_Set.Table, 7);


      Add_With_Viewport (Update_Tasks_Set.Scroll, Update_Tasks_Set.Table);
      Set_Focus_Hadjustment (Update_Tasks_Set.Table, Get_Hadjustment (
            Update_Tasks_Set.Scroll));
      Set_Focus_Vadjustment (Update_Tasks_Set.Table, Get_Vadjustment (
            Update_Tasks_Set.Scroll));


      Gtk_New (Update_Tasks_Set.Name, To_String(Lb_Task_Name(
               Current_Language)));
      Set_Alignment (Update_Tasks_Set.Name, 0.5, 0.5);
      Set_Padding (Update_Tasks_Set.Name, 0, 0);
      Set_Justify (Update_Tasks_Set.Name, Justify_Center);
      Set_Line_Wrap (Update_Tasks_Set.Name, False);
      Attach (Update_Tasks_Set.Table, Update_Tasks_Set.Name, 0, 1, 0,
         1,
         0, 0,
         0, 0);

      Gtk_New (Update_Tasks_Set.Processor, To_String(Lb_Processor(
               Current_Language)));
      Set_Alignment (Update_Tasks_Set.Processor, 0.5, 0.5);
      Set_Padding (Update_Tasks_Set.Processor, 0, 0);
      Set_Justify (Update_Tasks_Set.Processor, Justify_Center);
      Set_Line_Wrap (Update_Tasks_Set.Processor, False);
      Attach (Update_Tasks_Set.Table, Update_Tasks_Set.Processor, 1, 2,
         0,
         1,
         0, 0,
         0, 0);

      Gtk_New (Update_Tasks_Set.Start_Time, To_String(Lb_Start_Time(
               Current_Language)));
      Set_Alignment (Update_Tasks_Set.Start_Time, 0.5, 0.5);
      Set_Padding (Update_Tasks_Set.Start_Time, 0, 0);
      Set_Justify (Update_Tasks_Set.Start_Time, Justify_Center);
      Set_Line_Wrap (Update_Tasks_Set.Start_Time, False);
      Attach (Update_Tasks_Set.Table, Update_Tasks_Set.Start_Time, 2, 3, 0,
         1,
         0, 0,
         0, 0);

      Gtk_New (Update_Tasks_Set.Capacity, To_String(Lb_Capacity(
               Current_Language)));
      Set_Alignment (Update_Tasks_Set.Capacity, 0.5, 0.5);
      Set_Padding (Update_Tasks_Set.Capacity, 0, 0);
      Set_Justify (Update_Tasks_Set.Capacity, Justify_Center);
      Set_Line_Wrap (Update_Tasks_Set.Capacity, False);
      Attach (Update_Tasks_Set.Table, Update_Tasks_Set.Capacity, 3, 4, 0,
         1,
         0, 0,
         0, 0);

      Gtk_New (Update_Tasks_Set.Period, To_String(Lb_Period(
               Current_Language)));
      Set_Alignment (Update_Tasks_Set.Period, 0.5, 0.5);
      Set_Padding (Update_Tasks_Set.Period, 0, 0);
      Set_Justify (Update_Tasks_Set.Period, Justify_Center);
      Set_Line_Wrap (Update_Tasks_Set.Period, False);
      Attach (Update_Tasks_Set.Table, Update_Tasks_Set.Period, 4, 5, 0,
         1,
         0, 0,
         0, 0);


      Gtk_New (Update_Tasks_Set.Deadline, To_String(Lb_Deadline(
               Current_Language)));
      Set_Alignment (Update_Tasks_Set.Deadline, 0.5, 0.5);
      Set_Padding (Update_Tasks_Set.Deadline, 0, 0);
      Set_Justify (Update_Tasks_Set.Deadline, Justify_Center);
      Set_Line_Wrap (Update_Tasks_Set.Deadline, False);
      Attach (Update_Tasks_Set.Table, Update_Tasks_Set.Deadline, 5, 6, 0,
         1,
         0, 0,
         0, 0);

      Gtk_New (Update_Tasks_Set.Jitter, To_String(Lb_Jitter(
               Current_Language)));
      Set_Alignment (Update_Tasks_Set.Jitter, 0.5, 0.5);
      Set_Padding (Update_Tasks_Set.Jitter, 0, 0);
      Set_Justify (Update_Tasks_Set.Jitter, Justify_Center);
      Set_Line_Wrap (Update_Tasks_Set.Jitter, False);
      Attach (Update_Tasks_Set.Table, Update_Tasks_Set.Jitter, 6, 7, 0,
         1,
         0, 0,
         0, 0);

      Gtk_New (Update_Tasks_Set.Priority, To_String(Lb_Priority(
               Current_Language)));
      Set_Alignment (Update_Tasks_Set.Priority, 0.5, 0.5);
      Set_Padding (Update_Tasks_Set.Priority, 0, 0);
      Set_Justify (Update_Tasks_Set.Priority, Justify_Center);
      Set_Line_Wrap (Update_Tasks_Set.Priority, False);
      Attach (Update_Tasks_Set.Table, Update_Tasks_Set.Priority, 7, 8, 0,
         1,
         0, 0,
         0, 0);

      Gtk_New (Update_Tasks_Set.Priority, To_String(Lb_Offset_Act(
               Current_Language)));
      Set_Alignment (Update_Tasks_Set.Priority, 0.5, 0.5);
      Set_Padding (Update_Tasks_Set.Priority, 0, 0);
      Set_Justify (Update_Tasks_Set.Priority, Justify_Center);
      Set_Line_Wrap (Update_Tasks_Set.Priority, False);
      Attach (Update_Tasks_Set.Table, Update_Tasks_Set.Priority,  8, 11, 0,
         1,
         0, 0,
         0, 0);


      Gtk_New (Update_Tasks_Set.Priority, To_String(Lb_Offset_Val(
               Current_Language)));
      Set_Alignment (Update_Tasks_Set.Priority, 0.5, 0.5);
      Set_Padding (Update_Tasks_Set.Priority, 0, 0);
      Set_Justify (Update_Tasks_Set.Priority, Justify_Center);
      Set_Line_Wrap (Update_Tasks_Set.Priority, False);
      Attach (Update_Tasks_Set.Table, Update_Tasks_Set.Priority, 11, 14, 0,
         1,
         0, 0,
         0, 0);




      Reset_Iterator(Titerator);

      loop
         Current_Element(Sys.Tasks, A_Task, Titerator);

         Gtk_New (Update_Tasks_Set.Entries(Tasks_Range(Ligne-1)).Name);
         Set_Editable (Update_Tasks_Set.Entries(Tasks_Range(Ligne-1)).
            Name, True);
         Set_Max_Length (Update_Tasks_Set.Entries(Tasks_Range(Ligne-1)).
            Name, 0);
         Set_Text (Update_Tasks_Set.Entries(Tasks_Range(Ligne-1)).Name,
            To_String(A_Task.Name) );
         Set_Visibility (Update_Tasks_Set.Entries(Tasks_Range(Ligne-1)).
            Name, True);
         Attach (Update_Tasks_Set.Table, Update_Tasks_Set.Entries(
               Tasks_Range(Ligne-1)).Name, 0, 1,
            Ligne,
            Ligne+1,
            Expand or Fill, 0,
            0, 0);


         Initialize(Items);
         Index:=0;
         Current:=-1;
         Reset_Iterator(Piterator);

         loop
            Current_Element(Sys.Processors, A_Processor, Piterator);
            Items(Index):=A_Processor.Name;
            if(A_Task.Location=A_Processor.Name)
                  then
               Current:=Gint(Index);
            end if;

            Index:=Index+1;
            exit when Is_Last_Element(Sys.Processors, Piterator) = True;

            Next_Element(Sys.Processors, Piterator);
         end loop;


         Update_Tasks_Set.Entries(Tasks_Range(Ligne-1)).
            Clist_Processor_Menu:= Widget_Slist.Null_List;
         Build_Option_Menu (Update_Tasks_Set.Entries(Tasks_Range(Ligne-1)).
            Processor_Menu,
            Update_Tasks_Set.Entries(Tasks_Range(Ligne-1)).
            Clist_Processor_Menu, Items,
            Current, null);


         Attach (Update_Tasks_Set.Table, Update_Tasks_Set.Entries(
               Tasks_Range(Ligne-1)).Processor_Menu, 1, 2,
            Ligne,
            Ligne+1,
            Expand or Fill, 0,
            0, 0);



         Gtk_New (Update_Tasks_Set.Entries(Tasks_Range(Ligne-1)).
            Start_Time);
         Set_Editable (Update_Tasks_Set.Entries(Tasks_Range(Ligne-1)).
            Start_Time, True);
         Set_Max_Length (Update_Tasks_Set.Entries(Tasks_Range(Ligne-1)).
            Start_Time, 0);
         Set_Text (Update_Tasks_Set.Entries(Tasks_Range(Ligne-1)).
            Start_Time, A_Task.Start_Time'Img);
         Set_Visibility (Update_Tasks_Set.Entries(Tasks_Range(Ligne-1)).
            Start_Time, True);
         Attach (Update_Tasks_Set.Table, Update_Tasks_Set.Entries(
               Tasks_Range(Ligne-1)).Start_Time, 2, 3,
            Ligne,
            Ligne+1,
            Expand or Fill, 0,
            0, 0);

         Gtk_New (Update_Tasks_Set.Entries(Tasks_Range(Ligne-1)).Capacity);
         Set_Editable (Update_Tasks_Set.Entries(Tasks_Range(Ligne-1)).
            Capacity, True);
         Set_Max_Length (Update_Tasks_Set.Entries(Tasks_Range(Ligne-1)).
            Capacity, 0);
         Set_Text (Update_Tasks_Set.Entries(Tasks_Range(Ligne-1)).
            Capacity, A_Task.Capacity'Img);
         Set_Visibility (Update_Tasks_Set.Entries(Tasks_Range(Ligne-1)).
            Capacity, True);
         Attach (Update_Tasks_Set.Table, Update_Tasks_Set.Entries(
               Tasks_Range(Ligne-1)).Capacity, 3, 4,
            Ligne,
            Ligne+1,
            Expand or Fill, 0,
            0, 0);

         Gtk_New (Update_Tasks_Set.Entries(Tasks_Range(Ligne-1)).Period);
         Set_Editable (Update_Tasks_Set.Entries(Tasks_Range(Ligne-1)).
            Period, True);
         Set_Max_Length (Update_Tasks_Set.Entries(Tasks_Range(Ligne-1)).
            Period, 0);
         if (A_Task.Task_Type = Periodic_Type)
               then
            Set_Text (Update_Tasks_Set.Entries(Tasks_Range(Ligne-1)).
               Period, Periodic_Task_Ptr(
                  A_Task).Period'Img);
         else
            Set_Text (Update_Tasks_Set.Entries(Tasks_Range(Ligne-1)).
               Period, "0");
         end if;

         Set_Visibility (Update_Tasks_Set.Entries(Tasks_Range(Ligne-1)).
            Period, True);
         Attach (Update_Tasks_Set.Table, Update_Tasks_Set.Entries(
               Tasks_Range(Ligne-1)).Period, 4, 5,
            Ligne,
            Ligne+1,
            Expand or Fill, 0,
            0, 0);

         Gtk_New (Update_Tasks_Set.Entries(Tasks_Range(Ligne-1)).Deadline);
         Set_Editable (Update_Tasks_Set.Entries(Tasks_Range(Ligne-1)).
            Deadline, True);
         Set_Max_Length (Update_Tasks_Set.Entries(Tasks_Range(Ligne-1)).
            Deadline, 0);
         if (A_Task.Task_Type = Periodic_Type)
               then
            Set_Text (Update_Tasks_Set.Entries(Tasks_Range(Ligne-1)).
               Deadline, Periodic_Task_Ptr(A_Task).
               Deadline'Img);
         else
            Set_Text (Update_Tasks_Set.Entries(Tasks_Range(Ligne-1)).
               Deadline, Aperiodic_Task_Ptr(A_Task).
               Deadline'Img);
         end if;
         Set_Visibility (Update_Tasks_Set.Entries(Tasks_Range(Ligne-1)).
            Deadline, True);
         Attach (Update_Tasks_Set.Table, Update_Tasks_Set.Entries(
               Tasks_Range(Ligne-1)).Deadline, 5, 6,
            Ligne,
            Ligne+1,
            Expand or Fill, 0,
            0, 0);


         Gtk_New (Update_Tasks_Set.Entries(Tasks_Range(Ligne-1)).Jitter);
         Set_Editable (Update_Tasks_Set.Entries(Tasks_Range(Ligne-1)).
            Jitter, True);
         Set_Max_Length (Update_Tasks_Set.Entries(Tasks_Range(Ligne-1)).
            Jitter, 0);
         if (A_Task.Task_Type = Periodic_Type)
               then
            Set_Text (Update_Tasks_Set.Entries(Tasks_Range(Ligne-1)).
               Jitter, Periodic_Task_Ptr(A_Task).
               Jitter'Img);
         else
            Set_Text (Update_Tasks_Set.Entries(Tasks_Range(Ligne-1)).
               Jitter, "0");
         end if;
         Set_Visibility (Update_Tasks_Set.Entries(Tasks_Range(Ligne-1)).
            Jitter, True);
         Attach (Update_Tasks_Set.Table, Update_Tasks_Set.Entries(
               Tasks_Range(Ligne-1)).Jitter, 6, 7,
            Ligne,
            Ligne+1,
            Expand or Fill, 0,
            0, 0);

         Gtk_New (Update_Tasks_Set.Entries(Tasks_Range(Ligne-1)).Priority);
         Set_Editable (Update_Tasks_Set.Entries(Tasks_Range(Ligne-1)).
            Priority, True);
         Set_Max_Length (Update_Tasks_Set.Entries(Tasks_Range(Ligne-1)).
            Priority, 0);
         Set_Text (Update_Tasks_Set.Entries(Tasks_Range(Ligne-1)).
            Priority, A_Task.Priority'Img);
         Set_Visibility (Update_Tasks_Set.Entries(Tasks_Range(Ligne-1)).
            Priority, True);
         Attach (Update_Tasks_Set.Table, Update_Tasks_Set.Entries(
               Tasks_Range(Ligne-1)).Priority, 7, 8,
            Ligne,
            Ligne+1,
            Expand or Fill, 0,
            0, 0);

         Gtk_New (Update_Tasks_Set.Entries(Tasks_Range(Ligne-1)).
            Offset_Act1);
         Set_Editable (Update_Tasks_Set.Entries(Tasks_Range(Ligne-1)).
            Offset_Act1, True);
         Set_Max_Length (Update_Tasks_Set.Entries(Tasks_Range(Ligne-1)).
            Offset_Act1, 0);
         Set_Text (Update_Tasks_Set.Entries(Tasks_Range(Ligne-1)).
            Offset_Act1, A_Task.Offset(1).Activation'Img);
         Set_Visibility (Update_Tasks_Set.Entries(Tasks_Range(Ligne-1)).
            Offset_Act1, True);
         Attach (Update_Tasks_Set.Table, Update_Tasks_Set.Entries(
               Tasks_Range(Ligne-1)).Offset_Act1, 8, 9,
            Ligne,
            Ligne+1,
            Expand or Fill, 0,
            0, 0);

         Gtk_New (Update_Tasks_Set.Entries(Tasks_Range(Ligne-1)).
            Offset_Act2);
         Set_Editable (Update_Tasks_Set.Entries(Tasks_Range(Ligne-1)).
            Offset_Act2, True);
         Set_Max_Length (Update_Tasks_Set.Entries(Tasks_Range(Ligne-1)).
            Offset_Act2, 0);
         Set_Text (Update_Tasks_Set.Entries(Tasks_Range(Ligne-1)).
            Offset_Act2, A_Task.Offset(2).Activation'Img);
         Set_Visibility (Update_Tasks_Set.Entries(Tasks_Range(Ligne-1)).
            Offset_Act2, True);
         Attach (Update_Tasks_Set.Table, Update_Tasks_Set.Entries(
               Tasks_Range(Ligne-1)).Offset_Act2, 9, 10,
            Ligne,
            Ligne+1,
            Expand or Fill, 0,
            0, 0);

         Gtk_New (Update_Tasks_Set.Entries(Tasks_Range(Ligne-1)).
            Offset_Act3);
         Set_Editable (Update_Tasks_Set.Entries(Tasks_Range(Ligne-1)).
            Offset_Act3, True);
         Set_Max_Length (Update_Tasks_Set.Entries(Tasks_Range(Ligne-1)).
            Offset_Act3, 0);
         Set_Text (Update_Tasks_Set.Entries(Tasks_Range(Ligne-1)).
            Offset_Act3, A_Task.Offset(3).Activation'Img);
         Set_Visibility (Update_Tasks_Set.Entries(Tasks_Range(Ligne-1)).
            Offset_Act3, True);
         Attach (Update_Tasks_Set.Table, Update_Tasks_Set.Entries(
               Tasks_Range(Ligne-1)).Offset_Act3, 10, 11,
            Ligne,
            Ligne+1,
            Expand or Fill, 0,
            0, 0);

         Gtk_New (Update_Tasks_Set.Entries(Tasks_Range(Ligne-1)).
            Offset_Val1);
         Set_Editable (Update_Tasks_Set.Entries(Tasks_Range(Ligne-1)).
            Offset_Val1, True);
         Set_Max_Length (Update_Tasks_Set.Entries(Tasks_Range(Ligne-1)).
            Offset_Val1, 0);
         Set_Text (Update_Tasks_Set.Entries(Tasks_Range(Ligne-1)).
            Offset_Val1, A_Task.Offset(1).Value'Img);
         Set_Visibility (Update_Tasks_Set.Entries(Tasks_Range(Ligne-1)).
            Offset_Val1, True);
         Attach (Update_Tasks_Set.Table, Update_Tasks_Set.Entries(
               Tasks_Range(Ligne-1)).Offset_Val1, 11, 12,
            Ligne,
            Ligne+1,
            Expand or Fill, 0,
            0, 0);

         Gtk_New (Update_Tasks_Set.Entries(Tasks_Range(Ligne-1)).
            Offset_Val2);
         Set_Editable (Update_Tasks_Set.Entries(Tasks_Range(Ligne-1)).
            Offset_Val2, True);
         Set_Max_Length (Update_Tasks_Set.Entries(Tasks_Range(Ligne-1)).
            Offset_Val2, 0);
         Set_Text (Update_Tasks_Set.Entries(Tasks_Range(Ligne-1)).
            Offset_Val2, A_Task.Offset(2).Value'Img);
         Set_Visibility (Update_Tasks_Set.Entries(Tasks_Range(Ligne-1)).
            Offset_Val2, True);
         Attach (Update_Tasks_Set.Table, Update_Tasks_Set.Entries(
               Tasks_Range(Ligne-1)).Offset_Val2, 12, 13,
            Ligne,
            Ligne+1,
            Expand or Fill, 0,
            0, 0);

         Gtk_New (Update_Tasks_Set.Entries(Tasks_Range(Ligne-1)).
            Offset_Val3);
         Set_Editable (Update_Tasks_Set.Entries(Tasks_Range(Ligne-1)).
            Offset_Val3, True);
         Set_Max_Length (Update_Tasks_Set.Entries(Tasks_Range(Ligne-1)).
            Offset_Val3, 0);
         Set_Text (Update_Tasks_Set.Entries(Tasks_Range(Ligne-1)).
            Offset_Val3, A_Task.Offset(3).Value'Img);
         Set_Visibility (Update_Tasks_Set.Entries(Tasks_Range(Ligne-1)).
            Offset_Val3, True);
         Attach (Update_Tasks_Set.Table, Update_Tasks_Set.Entries(
               Tasks_Range(Ligne-1)).Offset_Val3, 13, 14,
            Ligne,
            Ligne+1,
            Expand or Fill, 0,
            0, 0);





         Ligne:=Ligne+1;
         exit when Is_Last_Element(Sys.Tasks, Titerator) = True;

         Next_Element(Sys.Tasks, Titerator);

      end loop;


      Gtk_New_Hseparator (Update_Tasks_Set.Hseparator);
      Attach (Update_Tasks_Set.Table, Update_Tasks_Set.
         Hseparator, 0, 14, Ligne, Ligne+1,
         Fill, Expand or Fill,
         0, 0);


      Gtk_New_Hbox (Update_Tasks_Set.Hbox, True, 0);
      Set_Border_Width (Update_Tasks_Set.Hbox, 6);
      Attach (Update_Tasks_Set.Table, Update_Tasks_Set.Hbox, 0, 14, Ligne+
         1, Ligne+
         2,
         Fill, Expand or Fill,
         0, 0);


      Gtk_New (Update_Tasks_Set.Ok_Button, "OK");
      Pack_Start (Update_Tasks_Set.Hbox, Update_Tasks_Set.Ok_Button, True,
         True, 0);

      Gtk_New_Vseparator (Update_Tasks_Set.Vseparator);
      Set_Sensitive (Update_Tasks_Set.Vseparator, False);
      Pack_Start (Update_Tasks_Set.Hbox, Update_Tasks_Set.Vseparator, True,
         True, 0);

      Gtk_New (Update_Tasks_Set.Cancel_Button, To_String(Lb_Cancel(
               Current_Language)));
      Pack_Start (Update_Tasks_Set.Hbox, Update_Tasks_Set.Cancel_Button,
         True, True, 0);


      Set_Flags (Update_Tasks_Set.Ok_Button, Can_Default);

      Button_Callback.Connect
         (Update_Tasks_Set.Ok_Button, "clicked",
         Button_Callback.To_Marshaller (On_Ok_Pressed'access));
      Button_Callback.Connect
         (Update_Tasks_Set.Cancel_Button, "clicked",
         Button_Callback.To_Marshaller (On_Cancel_Pressed'access));



   end Initialize;

end Update_Tasks_Set_Pkg;
