------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a real time simulator mainly designed for educational purpose. 
-- This program provides services to automaticly check temporal constraints 
-- of real time tasks.
--
-- Copyright (C) 2002   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the LIMI Team/EA 2215
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------




with Fixed_Strings;
use Fixed_Strings;

with Config;
use Config;

with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;


with Buffers;
use Buffers;



with Sets;


package Buffer_Set is


   Buffer_Not_Found       : exception;  
   Buffer_Already_Defined : exception;  
   Invalid_Parameter      : exception;  


   type Buffer_Descriptor is 
      record 
         Name     : Fixed_String;  
         Size     : Natural;  
         Location : Fixed_String;  
         Rules    : Buffer_Rules_Table;  
      end record; 


   type Buffer_Parameters is 
         (Size,     
          Location, 
          Rules); 


   package Generic_Buffer_Set is new Sets (
      Max_Element     => Config.Max_Buffers, 
      File_Descriptor => Buffer_Descriptor,  
      Element         => Buffer_Ptr);


   use Generic_Buffer_Set;

   type Buffers_Set is new Generic_Buffer_Set.Set with private; 


   subtype Buffers_Range is Generic_Buffer_Set.Element_Range;
   subtype Buffers_Iterator is   Generic_Buffer_Set.Iterator;




   procedure Fill_Descriptor (
         My_Buffers : in     Buffers_Set;       
         Desc       :    out Buffer_Descriptor; 
         A_Buffer   : in     Buffer_Ptr         ); 


   procedure Extract_Descriptor (
         My_Buffers : in     Buffers_Set;       
         Desc       : in     Buffer_Descriptor; 
         A_Buffer   :    out Buffer_Ptr         ); 


   procedure Dump (
         My_Buffers : in     Buffers_Set ); 


   procedure Add_Buffer (
         My_Buffers : in out Buffers_Set;       
         Name       : in     Unbounded_String;  
         Size       : in     Natural;           
         Location   : in     Unbounded_String;  
         Rules      : in     Buffer_Rules_Table ); 


   function Get (
         My_Buffers  : in     Buffers_Set;      
         Buffer_Name : in     Unbounded_String; 
         Param_Name  : in     Buffer_Parameters ) 
     return Unbounded_String; 

   procedure Set (
         My_Buffers  : in out Buffers_Set;       
         Buffer_Name : in     Unbounded_String;  
         Param_Name  : in     Buffer_Parameters; 
         Param_Value : in     Unbounded_String   ); 

   function Get (
         My_Buffers  : in     Buffers_Set;      
         Buffer_Name : in     Unbounded_String; 
         Param_Name  : in     Buffer_Parameters ) 
     return Natural; 

   procedure Set (
         My_Buffers  : in out Buffers_Set;       
         Buffer_Name : in     Unbounded_String;  
         Param_Name  : in     Buffer_Parameters; 
         Param_Value : in     Natural            ); 


private

   type Buffers_Set is new Generic_Buffer_Set.Set with null record; 




end Buffer_Set;
