------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a real time simulator mainly designed for educational purpose. 
-- This program provides services to automaticly check temporal constraints 
-- of real time tasks.
--
-- Copyright (C) 2002   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the LIMI Team/EA 2215
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------




with Config;
use Config;

with Tables;

with Time_Units;
use Time_Units;

with Task_Set;
use Task_Set;
use Task_Set.Generic_Task_Set;

with Dependencies;
use Dependencies;

with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;


package Buffers is


   type Rule is 
         (Norule,   
          Producer, 
          Consumer); 
   type Buffer_Rule is 
      record 
         The_Rule : Rule;  
         Size     : Natural;  
      end record; 

   procedure Put (
         B : in     Buffer_Rule ); 
   procedure Initialize (
         B :    out Buffer_Rule ); 

   package Buffer_Rules is new Tables(Buffer_Rule,
      Config.Max_Tasks,
      Put, Initialize);
   use Buffer_Rules;

   subtype Buffer_Rules_Table is Table;
   subtype Buffer_Rules_Range is Table_Range;

   --------------------------------------------------------

   type Buffer is new Dependency with 
      record 
         Name     : Unbounded_String   := To_Unbounded_String ("");  
         Location : Unbounded_String   := To_Unbounded_String ("");  
         Size     : Natural            := 0;  
         Rules    : Buffer_Rules_Table;  
      end record; 

   type Buffer_Ptr is access all Buffer'Class; 


   procedure Initialize (
         A_Buff : in out Buffer ); 
   procedure Finalize (
         A_Buff : in out Buffer ); 
   procedure Adjust (
         A_Buff : in out Buffer ); 


   procedure Dispatched_Dump (
         My_Buff : in     Buffer ); 



end Buffers;





