------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a real time simulator mainly designed for educational purpose. 
-- This program provides services to automaticly check temporal constraints 
-- of real time tasks.
--
-- Copyright (C) 2002   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the LIMI Team/EA 2215
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------





with Ada.Finalization;


package Dependencies is


   type Dependency_Type is 
         (Precedence_Dependency,   
          Buffer_Dependency,       
          Communication_Dependency); 

   type Dependency is abstract new Ada.Finalization.Controlled with 
      record 
         Dependence_Type : Dependency_Type;  
      end record; 
   type Dependency_Ptr is access all Dependency'Class; 


   procedure Initialize (
         A_Dep : in out Dependency ); 
   procedure Finalize (
         A_Dep : in out Dependency ); 
   procedure Adjust (
         A_Dep : in out Dependency ); 


   procedure Dump (
         My_Dep : in     Dependency'Class ); 


   procedure Dispatched_Dump (
         My_Dep : in     Dependency ) is 
   abstract;


   ----------------------------------------------------------------------

   type Precedence is new Dependency with null record; 
   type Precedence_Ptr is access all Precedence'Class; 


   procedure Initialize (
         A_Pred : in out Precedence ); 
   procedure Finalize (
         A_Pred : in out Precedence ); 
   procedure Adjust (
         A_Pred : in out Precedence ); 

   procedure Dispatched_Dump (
         My_Pred : in     Precedence ); 


end Dependencies;


