------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a real time simulator mainly designed for educational purpose. 
-- This program provides services to automaticly check temporal constraints 
-- of real time tasks.
--
-- Copyright (C) 2002   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the LIMI Team/EA 2215
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------




with Task_Set;
use Task_Set;
use Task_Set.Generic_Task_Set;

with Scheduler;


with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;




package Scheduler.Edf is


   type Edf_Scheduler is new Generic_Scheduler with private; 

   type Edf_Tcb is new Tcb with 
      record 
         Dynamic_Deadline : Natural;  
      end record; 
   type Edf_Tcb_Ptr is access all Edf_Tcb'Class; 





   procedure Initialize (
         A_Scheduler : in out Edf_Scheduler ); 

   procedure Finalize (
         A_Scheduler : in out Edf_Scheduler ); 

   procedure Adjust (
         A_Scheduler : in out Edf_Scheduler ); 


   function Dispatched_Build_Tcb (
         My_Scheduler : in     Edf_Scheduler; 
         A_Task       :        Task_Ptr       ) 
     return Tcb_Ptr; 

   procedure Dispatched_Check_Before_Scheduling (
         My_Scheduler   : in     Edf_Scheduler;   
         My_Tasks       : in     Tasks_Set;       
         Processor_Name : in     Unbounded_String ); 

   procedure Dispatched_Do_Election (
         My_Scheduler : in     Edf_Scheduler; 
         A_Tcb        : in     Tcb_Table;     
         At_Time      :        Natural;       
         Elected      : in out Tasks_Range;   
         No_Task      : in out Boolean        ); 


   procedure Dispatched_Bound_On_Processor_Utilization (
         My_Scheduler   : in     Edf_Scheduler;    
         My_Tasks       : in     Tasks_Set;        
         Processor_Name : in     Unbounded_String; 
         Result         : in out Double;           
         Msg            : in out Unbounded_String  ); 


   procedure Dispatched_Reset (
         A_Scheduler : in out Edf_Scheduler ); 


   procedure Dispatched_Response_Time (
         My_Scheduler   : in     Edf_Scheduler;    
         My_Tasks       : in out Tasks_Set;        
         Processor_Name : in     Unbounded_String; 
         Msg            : in out Unbounded_String; 
         Response_Time  :    out Response_Table    ); 


private

   type Edf_Scheduler is new Generic_Scheduler with null record; 

end Scheduler.Edf ;


