------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a real time simulator mainly designed for educational purpose. 
-- This program provides services to automaticly check temporal constraints 
-- of real time tasks.
--
-- Copyright (C) 2002   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the LIMI Team/EA 2215
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------





with Config;
use Config;

with Translate;
use Translate;

with Tasks;
use Tasks;

with Scheduler;
use Scheduler;


with Time_Units;
use Time_Units;
use Time_Units.A_Time_Unit;

with Unbounded_Strings;
use Unbounded_Strings;




package body Scheduler.Llf is


   procedure Initialize (
         A_Scheduler : in out Llf_Scheduler ) is 
   begin
      Reset(A_Scheduler);
   end Initialize;

   procedure Finalize (
         A_Scheduler : in out Llf_Scheduler ) is 
   begin
      null;
   end Finalize;

   procedure Adjust (
         A_Scheduler : in out Llf_Scheduler ) is 
   begin
      null;
   end Adjust;


   procedure Dispatched_Reset (
         A_Scheduler : in out Llf_Scheduler ) is 
   begin
      A_Scheduler.Name:= Least_Laxity_First;
   end Dispatched_Reset;



   function Dispatched_Build_Tcb (
         My_Scheduler : in     Llf_Scheduler; 
         A_Task       :        Task_Ptr       ) 
     return Tcb_Ptr is 

      A_Tcb : Tcb_Ptr;  
   begin
      A_Tcb:=new Llf_Tcb;

      A_Tcb.Tsk:=A_Task;
      A_Tcb.Activation:=1;
      A_Tcb.Wake_Up_Time:=A_Task.Start_Time;
      A_Tcb.Rest_Of_Capacity:=A_Task.Capacity;

      if A_Task.Task_Type = Periodic_Type
            then
         if(A_Task.Deadline /= 0)
               then
            Llf_Tcb_Ptr(A_Tcb).Dynamic_Deadline:=
               A_Task.Deadline+A_Tcb.Wake_Up_Time;
         else
            Llf_Tcb_Ptr(A_Tcb).Dynamic_Deadline:=Periodic_Task_Ptr(
               A_Task).Period+A_Tcb.Wake_Up_Time;
         end if;
      else
         Llf_Tcb_Ptr(A_Tcb).Dynamic_Deadline:=A_Task.Deadline
            +A_Tcb.Wake_Up_Time;
      end if;

      return A_Tcb;
   end Dispatched_Build_Tcb;


   procedure Dispatched_Check_Before_Scheduling (
         My_Scheduler   : in     Llf_Scheduler;   
         My_Tasks       : in     Tasks_Set;       
         Processor_Name : in     Unbounded_String ) is 

   begin
      null;
   end Dispatched_Check_Before_Scheduling;


   procedure Dispatched_Do_Election (
         My_Scheduler : in     Llf_Scheduler; 
         A_Tcb        : in     Tcb_Table;     
         At_Time      :        Natural;       
         Elected      : in out Tasks_Range;   
         No_Task      : in out Boolean        ) is 

      Smallest_Laxity : Natural     := Natural'Last;  
      Laxity          : Natural;  
      I               : Tasks_Range := 0;  
   begin

      loop
         Laxity:=Llf_Tcb_Ptr(A_Tcb(I)).Dynamic_Deadline - A_Tcb(I).
            Rest_Of_Capacity;

         if (A_Tcb(I).Wake_Up_Time <= At_Time) and
               (Laxity < Smallest_Laxity) and
               Check_Offset(A_Tcb(I), At_Time) and
               (A_Tcb(I).Rest_Of_Capacity /= 0)
               then
            Smallest_Laxity:=Laxity;
            Elected:=I;
         end if;

         I:=I+1;
         exit when A_Tcb(I)=null;
      end loop;


      -- last time unit : compute dynamic deadline for periodic task
      --
      if (A_Tcb(Elected).Rest_Of_Capacity = 1)
            and (A_Tcb(Elected).Tsk.Task_Type = Periodic_Type)
            then
         Llf_Tcb_Ptr(A_Tcb(Elected)).Dynamic_Deadline:=
            Llf_Tcb_Ptr(A_Tcb(Elected)).Dynamic_Deadline
            +Periodic_Task_Ptr(A_Tcb(Elected).Tsk).Period;
      end if;


      if Smallest_Laxity= Natural'Last
            then
         No_Task:=True;
      else
         No_Task:=False;
      end if;

   end Dispatched_Do_Election;



   procedure Dispatched_Bound_On_Processor_Utilization (
         My_Scheduler   : in     Llf_Scheduler;    
         My_Tasks       : in     Tasks_Set;        
         Processor_Name : in     Unbounded_String; 
         Result         : in out Double;           
         Msg            : in out Unbounded_String  ) is 
   begin
      Result:=1.0;
      Msg:=To_Unbounded_String(" (") & Lb_See(Current_Language) &
         To_Unbounded_String("[1], page 8, ") & Lb_Theorem(
         Current_Language) &
         To_Unbounded_String("2). ") & Unbounded_Lf;

   end Dispatched_Bound_On_Processor_Utilization;



   procedure Dispatched_Response_Time (
         My_Scheduler   : in     Llf_Scheduler;    
         My_Tasks       : in out Tasks_Set;        
         Processor_Name : in     Unbounded_String; 
         Msg            : in out Unbounded_String; 
         Response_Time  :    out Response_Table    ) is 
   begin

      -- check if tasks are periodics
      Periodic_Control(My_Scheduler, My_Tasks, Processor_Name);

      -- check start time
      Start_Time_Control(My_Scheduler, My_Tasks, Processor_Name);

      -- check offset
      Offset_Control(My_Scheduler, My_Tasks, Processor_Name);


   end Dispatched_Response_Time;

end Scheduler.Llf ;


