------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a real time simulator mainly designed for educational purpose. 
-- This program provides services to automaticly check temporal constraints 
-- of real time tasks.
--
-- Copyright (C) 2002   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the LIMI Team/EA 2215
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------





with Config;
use Config;

with Translate;
use Translate;

with Tasks;
use Tasks;

with Scheduler;
use Scheduler;
use Scheduler.Double_Tasks_Parameters;

with Double_Util;

with Time_Units;
use Time_Units;
use Time_Units.A_Time_Unit;

with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;




with Unbounded_Strings;
use  Unbounded_Strings;



package body Scheduler.Rm is


   procedure Initialize (
         A_Scheduler : in out Rm_Scheduler ) is 
   begin
      Reset(A_Scheduler);
   end Initialize;

   procedure Finalize (
         A_Scheduler : in out Rm_Scheduler ) is 
   begin
      null;
   end Finalize;

   procedure Adjust (
         A_Scheduler : in out Rm_Scheduler ) is 
   begin
      null;
   end Adjust;

   procedure Dispatched_Reset (
         A_Scheduler : in out Rm_Scheduler ) is 
   begin
      A_Scheduler.Name:= Rate_Monotonic;
   end Dispatched_Reset;

function dispatched_build_tcb(
         My_Scheduler : in     rm_Scheduler;
        a_task : task_ptr)
        return tcb_ptr is 

a_tcb : tcb_ptr;
begin
	a_tcb:=new tcb;

            a_Tcb.tsk:=A_Task;
            a_Tcb.Activation:=1;
            a_Tcb.Wake_Up_Time:=A_Task.Start_Time;
            a_Tcb.Rest_Of_Capacity:=A_Task.Capacity;

	return a_tcb;
end dispatched_build_tcb;


procedure dispatched_check_before_scheduling(
         My_Scheduler : in     rm_Scheduler;
         My_Tasks       : in Tasks_Set;
         Processor_Name : in     Unbounded_String
        ) is 

begin

      periodic_control(my_scheduler, My_Tasks, Processor_Name);


      -- check if deadlines are equal  than period

      if (not Have_Deadlines_Equal_Than_Periods(My_Tasks, Processor_Name))
            then
         raise Task_Must_Have_Period_Equal_To_Deadline;
      end if;




end dispatched_check_before_scheduling;


procedure dispatched_do_election (
         My_Scheduler : in     rm_Scheduler;
        A_tcb : in tcb_table;
		at_time : natural;
                elected : in out tasks_range;
                no_task : in out boolean) is 

smallest_period : natural := natural'last;
i : tasks_range :=0;
begin

	loop
            if (a_Tcb(I).Wake_Up_Time <= at_Time) and
                  (periodic_task_ptr(a_Tcb(I).tsk).Period < Smallest_Period) and
                  Check_Offset(a_tcb(I), at_time) and
                  (a_Tcb(I).Rest_Of_Capacity /= 0)
                  then
               Smallest_Period:=periodic_task_ptr(a_Tcb(I).tsk).Period;
               Elected:=I;
            end if;

	i:=i+1;
	exit when a_tcb(i)=null;
         end loop;

if smallest_period = natural'last
	then no_task:=true;
	else no_task:=false;
end if;

end dispatched_do_election;




   procedure Set_Priority_According_To_Rm (
         My_Scheduler   : in     Rm_Scheduler;    
         My_Tasks       : in out Tasks_Set;       
         Processor_Name : in     Unbounded_String ) is 

      Iterator1    : Iterator;  
      Task1        : Task_Ptr;  
      Current_Prio : Priority_Range := High_Priority;  

   begin

      Sort(My_Tasks, Increasing_Period'access);

      Reset_Iterator(Iterator1);

      loop
         Current_Element(My_Tasks, Task1, Iterator1);

         if (Task1.Location = Processor_Name)
               then

            Task1.Priority:=Current_Prio;
            Current_Prio:=Current_Prio-1;
         end if;

         exit when Is_Last_Element(My_Tasks, Iterator1) = True;
         Next_Element(My_Tasks, Iterator1);

      end loop;
   end Set_Priority_According_To_Rm;




   procedure Dispatched_Response_Time (
         My_Scheduler   : in     Rm_Scheduler;     
         My_Tasks       : in out Tasks_Set;        
         Processor_Name : in     Unbounded_String; 
         Msg            : in out Unbounded_String; 
         Response_Time  :    out Response_Table    ) is 


tmp : tasks_set;
      Iterator1,  
      Iterator2   : Iterator;  
      Task1,  
      Task2, task_tmp       : Task_Ptr;  
      Number      : Response_Range;  
      Rn,  
      Rn1         : Double;  

   begin


	-- check if tasks are periodics 
      periodic_control(my_scheduler, My_Tasks, Processor_Name);

	-- check start time
      start_time_control(my_scheduler, My_Tasks, Processor_Name);

       -- check offset
      offset_control(my_scheduler, My_Tasks, Processor_Name);


      -- check if deadlines are equal  than period
      if (not Have_Deadlines_Equal_Than_Periods(My_Tasks, Processor_Name))
            then
         raise Task_Must_Have_Period_Equal_To_Deadline;
      end if;


      -- Check processor utilization first
      if (Processor_Utilization_Over_Period(My_Scheduler, My_Tasks,
               Processor_Name) > 1.0)
            then
         raise Processor_Utilization_Exceeded;
      end if;



      Msg:=To_Unbounded_String(" (") & Lb_See(Current_Language) &
         To_Unbounded_String("[2], page 3, ") & Lb_Equation(
         Current_Language)
         & To_Unbounded_String("4). ");


-- We will change priority task to compute response time
-- so, we will use a temporary task set 

      Reset_Iterator(Iterator1);
      loop


         Current_Element(my_tasks, Task1, Iterator1);
         if (Task1.Location = Processor_Name)
               then
		task_tmp:= new periodic_task;
		task_tmp.all:=task1.all;
		add(tmp,task_tmp);
	end if;

                  exit when Is_Last_Element(my_tasks, Iterator1) = True;
                  Next_Element(my_tasks, Iterator1);

	end loop;


      -- Now, compute response time
      Set_Priority_According_To_Rm(My_Scheduler, tmp, Processor_Name);


      Reset_Iterator(Iterator1);
      Number:=0;

      loop


         Current_Element(tmp, Task1, Iterator1);

         -- compute response time for a given task
         --

            Response_Time(Number).Task_Item:=Task1;
            Rn:=-0.1;
            Rn1:=0.0;

            while Rn1 /= Rn  loop

               Reset_Iterator(Iterator2);

               Rn:=Rn1;
               Rn1:=Double(Task1.Capacity+Task1.Blocking_Time);

               loop
                  Current_Element(tmp, Task2, Iterator2);

                  if (Task2.Priority > Task1.Priority)
                        then
                     Rn1:=Rn1+
                        Double_Util.Ceiling((Rn+Double(Periodic_Task_Ptr(
                                 Task2).Jitter))/Double(Periodic_Task_Ptr(
                              Task2).Period))*Double(Task2.Capacity) ;
                  end if;

                  exit when Is_Last_Element(tmp, Iterator2) = True;
                  Next_Element(tmp, Iterator2);

               end loop;


            end loop;


            Response_Time(Number).Data:=Rn1+Double(Periodic_Task_Ptr(
                  Task1).Jitter);
            Number:=Number+1;


         exit when Is_Last_Element(tmp, Iterator1) = True;
         Next_Element(tmp, Iterator1);


      end loop;


   end Dispatched_Response_Time;


   procedure Dispatched_Bound_On_Processor_Utilization (
         My_Scheduler   : in     Rm_Scheduler;     
         My_Tasks       : in     Tasks_Set;        
         Processor_Name : in     Unbounded_String; 
         Result         : in out Double;           
         Msg            : in out Unbounded_String  ) is 

      Nb : Double := Double (Get_Number_Of_Task_From_Processor (My_Tasks,
        Processor_Name));  
   begin
      Msg:=To_Unbounded_String(" (") & Lb_See(Current_Language) &
         To_Unbounded_String("[1], page 16, ") & Lb_Theorem(
         Current_Language) &
         To_Unbounded_String("8). ") & Unbounded_Lf;

      Result:=Nb*( Pow(2.0,(1.0/Nb)) - 1.0);
   end Dispatched_Bound_On_Processor_Utilization;



end Scheduler.Rm ;


