------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a real time simulator mainly designed for educational purpose. 
-- This program provides services to automaticly check temporal constraints 
-- of real time tasks.
--
-- Copyright (C) 2002   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the LIMI Team/EA 2215
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------




with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;

with Task_Set;
use Task_Set;
use Task_Set.Generic_Task_Set;

with Scheduler;
use Scheduler;
use Scheduler.Double_Tasks_Parameters;

with Scheduler.Rm;
use Scheduler.Rm;

with Scheduler.Dm;
use Scheduler.Dm;

with Text_Io;
use Text_Io;

with Offsets;
use Offsets;

procedure Response is 


   Sched1 : Rm_Scheduler;  
   Sched2 : Dm_Scheduler;  
   Tasks1 : Tasks_Set;  
   Tasks2 : Tasks_Set;  
   Rt     : Response_Table;  
   Cpu    : Unbounded_String := To_Unbounded_String ("cpu");  
   Offset : Offset_Table;  
   Dummy  : Unbounded_String;  

begin

   Initialize(Offset);

   Add_Task(Tasks1, To_Unbounded_String("T1"), Cpu , 0, 3, 7, 0, 0, 0, 1,
      Offset);
   Add_Task(Tasks1, To_Unbounded_String("T2"), Cpu , 0, 3,  12, 0, 0, 0, 1,
      Offset);
   Add_Task(Tasks1, To_Unbounded_String("T3"), Cpu , 0, 5, 20, 0, 0, 0, 1,
      Offset);


   Set_Priority_According_To_Rm(Sched1, Tasks1, Cpu);
   Dump(Tasks1);

   Compute_Response_Time(Sched1, Tasks1, Cpu, Dummy, Rt);
   Put_Line("TR should be : TR1 = 3 ; TR2 = 6 ; TR3 = 20");

   Dump(Tasks1);

   for I in 0 .. Response_Range(Get_Number_Of_Elements(Tasks1) - 1) loop
      Dump(Rt(I));
   end loop;

   New_Line;
   New_Line;


   Add_Task(Tasks2, To_Unbounded_String("T2"), Cpu , 0, 60,  150, 160, 0,
      20, 1, Offset);
   Add_Task(Tasks2, To_Unbounded_String("T3"), Cpu , 0, 60, 350, 350, 0, 0,
      1, Offset);
   Add_Task(Tasks2, To_Unbounded_String("T1"), Cpu , 0, 40, 100, 100, 0, 0,
      1, Offset);


   Set_Priority_According_To_Dm(Sched2, Tasks2, Cpu);
   Dump(Tasks2);

   Compute_Response_Time(Sched2, Tasks2, Cpu, Dummy, Rt);
   Put_Line("TR should be : TR1 = 40 ; TR2 = 160  ; TR3 = ??");

   Dump(Tasks2);

   for I in 0 .. Response_Range(Get_Number_Of_Elements(Tasks1) - 1) loop
      Dump(Rt(I));
   end loop;


end Response;
