------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a real time simulator mainly designed for educational purpose. 
-- This program provides services to automaticly check temporal constraints 
-- of real time tasks.
--
-- Copyright (C) 2002   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the LIMI Team/EA 2215
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------




with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;


with Ada.Numerics.Aux;
use Ada.Numerics.Aux;

with Task_Set;
use Task_Set;
use Task_Set.Generic_Task_Set;

with Scheduler;
use Scheduler;
use Scheduler.Double_Tasks_Parameters;

with Scheduler.Dm;
use Scheduler.Dm;

with Text_Io;
use Text_Io;

with Offsets;
use Offsets;


procedure Tindell1 is 


   Sched1      : Dm_Scheduler;  
   Tasks1      : Tasks_Set;  
   Response_M1,  
   Response_M2 : Double;  
   Initial_M1,  
   Initial_M2  : Double;  
   M1          : Double           := 0.0;  
   M2          : Double           := 0.0;  
   Rta,  
   Rta2        : Response_Table;  
   Rtb,  
   Rtb2        : Response_Table;  
   Offset      : Offset_Table;  
   Cpua        : Unbounded_String := To_Unbounded_String ("cpua");  
   Cpub        : Unbounded_String := To_Unbounded_String ("cpub");  
   Dummy       : Unbounded_String;  


   procedure Compute is 

   begin
      Compute_Response_Time(Sched1, Tasks1, Cpua, Dummy, Rta);
      Compute_Response_Time(Sched1, Tasks1, Cpub, Dummy, Rtb);

      for I in 0 .. Response_Range(Get_Number_Of_Task_From_Processor(
               Tasks1, Cpua) - 1) loop
         Dump(Rta(I));
      end loop;
      for I in 0 .. Response_Range(Get_Number_Of_Task_From_Processor(
               Tasks1, Cpub) - 1) loop
         Dump(Rtb(I));
      end loop;
      Put_Line(" m1 : " & Response_M1'Img);
      Put_Line(" m2 : " & Response_M2'Img);

      New_Line;
      New_Line;
   end Compute;

begin

   Initialize(Offset);

   -- task's parameters
   Add_Task(Tasks1, To_Unbounded_String("Ta1"), Cpua , 0, 1, 20, 0, 0, 0,
      1, Offset);
   Add_Task(Tasks1, To_Unbounded_String("Tb1"), Cpub , 0, 3,  20, 0, 0, 0,
      1, Offset);
   Add_Task(Tasks1, To_Unbounded_String("Ta2"), Cpua , 0, 1, 10, 0, 0, 0,
      1, Offset);
   Add_Task(Tasks1, To_Unbounded_String("Tb2"), Cpub , 0, 3,  10, 0, 0, 0,
      1, Offset);

   -- message's parameters
   Initial_M1:=10.0;
   Initial_M2:=2.0;
   Response_M1:=Initial_M1;
   Response_M2:=Initial_M2;

   Dump(Tasks1);
   New_Line;
   New_Line;

   Compute;

   New_Line;
   New_Line;

   while ((Rta2 /= Rta) and (Rtb2 /= Rtb)
         and (M1 /= Response_M1) and (M2 /= Response_M2) ) loop

      M1:=Response_M1;
      M2:=Response_M2;
      Rta2:=Rta;
      Rtb2:=Rtb;

      for I in 0 .. Response_Range(Get_Number_Of_Elements(Tasks1) - 1) loop
         if(Rta(I).Task_Item.Name = "Ta1")
               then
            Response_M1:=Initial_M1+Rta(I).Data;
            Task_Set.Set(Tasks1, To_Unbounded_String("Tb1"), Jitter,
               Natural(Response_M1));
         end if;
         if(Rtb(I).Task_Item.Name = "Tb2")
               then
            Response_M2:=Initial_M2+Rtb(I).Data;
            Task_Set.Set(Tasks1, To_Unbounded_String("Ta2"), Jitter,
               Natural(Response_M2));
         end if;
      end loop;


      Dump(Tasks1);
      Compute;


   end loop;

end Tindell1;
