------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Cheddar is a real time simulator mainly designed for educational purpose. 
-- This program provides services to automaticly check temporal constraints 
-- of real time tasks.
--
-- Copyright (C) 2002   Frank Singhoff, Jerome Legrand
-- Cheddar is developped by the LIMI Team/EA 2215
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
--
--
-- Contact : F. Singhoff (singhoff@univ-brest.fr)
--           J. Legrand (jerome.legrand@univ-brest.fr)
------------------------------------------------------------------------------
------------------------------------------------------------------------------



with Ada.Strings.Unbounded;
use Ada.Strings.Unbounded;



with Config;
use Config;


package Translate  is

   pragma Elaborate_Body;

   type Labels is array (Languages'range) of Unbounded_String; 


   ------------------------------------------------------------------------------
   --     Labels common on every languages
   ------------------------------------------------------------------------------

   Lb_Point,  
   Lb_Minus,  
   Lb_Colon : Unbounded_String;  


   ------------------------------------------------------------------------------
   --             Widgets labels
   ------------------------------------------------------------------------------
   -- Common
   Lb_Theorem,  
   Lb_Cancel,  
   Lb_See,  
   Lb_Equation,  
   Lb_No_Implem : Labels;  


   -- Root window
   --
   Lb_Root_Title,  
   Lb_File,  
   Lb_New,  
   Lb_Open,  
   Lb_Save,  
   Lb_Save_As,  
   Lb_Exit,  
   Lb_Edit,  
   Lb_View,  
   Lb_Open_File,  
   Lb_Save_File,  
   Lb_Tools,  
   Lb_Clear,  
   Lb_Draw,  
   Lb_Check,  
   Lb_Help,  
   Lb_Manual,  
   Lb_Ref,  
   Lb_About,  
   Lb_About_Text : Labels;  


   -- Update widgets
   --
   Lb_Update_Task,  
   Lb_Update_Processor : Labels;  


   -- Tasks parameters
   --
   Lb_Invalid_Priority,  
   Lb_Mandatory,  
   Lb_Already_Defined,  
   Lb_Must_Be,  
   Lb_Must_Be_Numeric,  
   Lb_Capacity,  
   Lb_Task_Name,  
   Lb_Jitter,  
   Lb_Period,  
   Lb_Deadline,  
   Lb_Priority,  
   Lb_Start_Time,  
   Lb_Offset_Val,  
   Lb_Offset_Act,  
   Lb_Values,  
   Lb_Add_Task         : Labels;  


   -- Tasks and processors list
   --
   Lb_Tasks_List,  
   Lb_Processors_List : Labels;  


   -- Processors parameters
   --
   Lb_Processor_Name,  
   Lb_Processor,  
   Lb_Scheduler,  
   Lb_Option,  
   Lb_Preemptive,  
   Lb_Not_Preemptive,  
   Lb_Add_Processor  : Labels;  


   -- Question widget
   --
   Lb_Parameter,  
   Lb_Schedule_From_To,  
   Lb_Delete_Task,  
   Lb_Delete_Processor,  
   Lb_Give_Task_Name,  
   Lb_Give_Processor_Name : Labels;  

   -- Message widget
   --
   Lb_Processor_Not_Found,  
   Lb_Task_Not_Found,  
   Lb_Define_Tasks_Before,  
   Lb_Define_Processors_Before : Labels;  


   ------------------------------------------------------------------------------
   --    Messages produced by call_scheduling.adb 
   ------------------------------------------------------------------------------
   Lb_Free_Unit_Time,  
   Lb_Utilization_Bound,  
   Lb_Utilization_With_Period,  
   Lb_Utilization_With_Deadline,  
   Lb_Task_Response_Time,  
   Lb_Scheduling_Period,  
   Lb_No_Missed_Deadline,  
   Lb_Number_Of_Preemption,  
   Lb_Compute_Error_1,  
   Lb_Compute_Error_2,  
   Lb_Compute_Error_3,  
   Lb_Compute_Error_4,  
   Lb_Compute_Error_5,  
   Lb_Compute_Error_6,  
   Lb_Compute_Error_7,  
   Lb_Compute_Error_8,  
   Lb_Compute_Error_9,  
   Lb_Compute_Error_10,  
   Lb_Compute_Error_11,  
   Lb_Compute_Error_12,  
   Lb_Compute_Error_14,  
   Lb_Compute_Error_15,  
   Lb_Compute_Error_13          : Labels;  


   ------------------------------------------------------------------------------
   --    Messages produced by tasks.adb
   ------------------------------------------------------------------------------
   Lb_Check_Deadline,  
   Lb_Periodic,  
   Lb_Aperiodic,  
   Lb_Completion_Time : Labels;  


   ------------------------------------------------------------------------------
   --    Messages produced by project.adb, open and save widgets
   ------------------------------------------------------------------------------
   Lb_Can_Not_Open,  
   Lb_Can_Not_Save,  
   Lb_Not_Saved    : Labels;  


   ------------------------------------------------------------------------------
   --    Messages of the draw buffers widgets
   ------------------------------------------------------------------------------
   Lb_Buffer_Utilization : Labels;  


end Translate;

